/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.JSHelper.Options;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.util.HashMap;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;

import com.sos.JSHelper.Options.JSOptionValueList;
import com.sos.JSHelper.Options.JSOptionsClass;

/**
 * \class JSOptionValueListTest
 *
 * \brief JSOptionValueListTest - Test fr die Klasse JSOptionValueList
 *
 * \details
 * Testet die Klasse JSOptionValueList
 *
 * \section JSOptionValueListTest.java_intro_sec Introduction
 *
 * \section JSOptionValueListTest.java_samples Some Samples
 *
 * \code
 *   .... code goes here ...
 * \endcode
 *
 * <p style="text-align:center">
 * <br />---------------------------------------------------------------------------
 * <br /> APL/Software GmbH - Berlin
 * <br />##### generated by ClaviusXPress (http://www.sos-berlin.com) #########
 * <br />---------------------------------------------------------------------------
 * </p>
 * \author EQALS
 * \version 11.05.2009
 * \see reference
 *
 * Created on 11.05.2009 11:57:27
 */
 
public class JSOptionValueListTest {

	@SuppressWarnings("unused")
	private final String			conClassName	= "JSOptionValueListTest";
	private HashMap<String, String>	settings		= null;
	private JSOptionsClass			objOptions		= null;

	private String					strKey			= "sql_statement";
	private String			 		strDescr		= "SQL Statements";
	private String					strDelimiter	= ";";
	private String[]				strStatements	= new String[10];
	private JSOptionValueList		optionValueList	= null;

	public JSOptionValueListTest() {
		//
	}

	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
	}

	@AfterClass
	public static void tearDownAfterClass() throws Exception {
	}

	@Before
	public void setUp() throws Exception {
		settings = new HashMap<String, String>();
		objOptions = new JSOptionsClass(settings);
	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	/**
	 * Testet eine leere Liste
	 */
	public void testConcatenatedValue1() throws Exception {
		optionValueList = new JSOptionValueList(objOptions, strKey, strDescr + "1", "", true);
		assertEquals("ConcatenatedValue::Empty", "", optionValueList.concatenatedValue(strDelimiter));

	}

	@Test
	/**
	 * Test Kein anhngen eines Semikolons
	 * Test Einfgen nicht nummerierter Key
	 * Ein einziges SQL-Statement endet nicht mit einem ";"
	 */
	public void testConcatenatedValue2() throws Exception {
		strStatements[0] = "SELECT * FROM HALLO";
		settings.put(strKey, strStatements[0]);
		optionValueList = new JSOptionValueList(objOptions, strKey, strDescr + "2", "", true);
		assertEquals("ConcatenatedValue::Single SQL Stmt1",
				strStatements[0],
				optionValueList.concatenatedValue(strDelimiter));

	}

	@Test
	/**
	 * Test Keine Verdopplung des Semikolons
	 * Ein einziges SQL-Statement endet mit einem ";".
	 */
	public void testConcatenatedValue3() throws Exception {
		strStatements[0] = "SELECT * FROM HALLO;";
		settings.put(strKey, strStatements[0]);
		optionValueList = new JSOptionValueList(objOptions, strKey, strDescr + "3", "", true);
		assertEquals("ConcatenatedValue::Single SQL Stmt2",
				strStatements[0],
				optionValueList.concatenatedValue(strDelimiter));

	}

	@Test
	/**
	 * Test: Keine Verdopplung des Semikolons bei mehreren Keys
	 * Test: Einfgen nicht nummerierter und nummerierter Key
	 * Zwei SQL-Statements in zwei Settings
	 * Erstes SQL-Statement endet mit einem ";".
	 * Key1 ohne Nummerierung, Key2 Nummer 1
	 */
	public void testConcatenatedValue4() throws Exception {
		strStatements[0] = "SELECT * FROM HALLO;SELECT ABC FROM TABELLE1;";
		strStatements[1] = "SELECT * FROM TABELLE2;";
		settings.put(strKey, strStatements[0]);
		settings.put(strKey + "1", strStatements[1]);
		optionValueList = new JSOptionValueList(objOptions, strKey, strDescr + "4", "", true);
		assertEquals("ConcatenatedValue::Multiple SQL Stmt1",
				strStatements[0] + strStatements[1],
				optionValueList.concatenatedValue(strDelimiter));

	}

	@Test
	/**
	 * Test: Einfgen des Semikolons bei mehreren Keys
	 * Zwei SQL-Statements in zwei Settings.
	 * Wie Fall 4, aber erstes SQL-Statement endet nicht mit einem ";"
	 * Key1 ohne Nummerierung, Key2 Nummer 1
	 */
	public void testConcatenatedValue5() throws Exception {
		strStatements[0] = "SELECT * FROM HALLO;SELECT ABC FROM TABELLE1";
		strStatements[1] = "SELECT * FROM TABELLE2;";
		settings.put(strKey, strStatements[0]);
		settings.put(strKey + "1", strStatements[1]);
		optionValueList = new JSOptionValueList(objOptions, strKey, strDescr + "5", "", true);
		assertEquals("ConcatenatedValue::Multiple SQL Stmt2",
				strStatements[0] + strDelimiter + strStatements[1],
				optionValueList.concatenatedValue(strDelimiter));

	}

	@Test
	/**
	 * Test: Alle Keys nummeriert
	 */
	public void testConcatenatedValue6() throws Exception {
		strStatements[0] = "SELECT * FROM HALLO;SELECT ABC FROM TABELLE1;";
		strStatements[1] = "SELECT * FROM TABELLE222";
		strStatements[2] = "SELECT * FROM TABELLE4711;";

		settings.put(strKey + "1", strStatements[0]);
		settings.put(strKey + "2", strStatements[1]);
		settings.put(strKey + "3", strStatements[2]);
		optionValueList = new JSOptionValueList(objOptions, strKey, strDescr + "6", "", true);
		assertEquals("ConcatenatedValue::Multiple SQL Stmt2b",
				strStatements[0] + strStatements[1] + strDelimiter + strStatements[2],
				optionValueList.concatenatedValue(strDelimiter));

	}

	@Test
	/**
	 * Test: Lcke in der Numerierung
	 */
	public void testConcatenatedValue7() throws Exception {

		strStatements[0] = "SELECT * FROM HALLO;SELECT ABC FROM TABELLE1;";
		strStatements[1] = "SELECT * FROM TABELLE222;";
		strStatements[2] = "SELECT SPALTE1 FROM TABELLE5;";
		settings.put(strKey + "1", strStatements[0]);
		//	settings.put(strKey + "2", strStatements[1]);
		settings.put(strKey + "3", strStatements[2]);
		optionValueList = new JSOptionValueList(objOptions, strKey, strDescr + "7", "", true);
		assertEquals("ConcatenatedValue::Multiple SQL Stmt4", strStatements[0], optionValueList.concatenatedValue(strDelimiter));
	}

	@Test
	@Ignore
	public void testAppendValue() throws Exception {
		JSOptionValueList objVL = new JSOptionValueList(objOptions, "ValueList", "Descr", "SELECT * FROM HALLO", "", false);
		assertTrue("contains SELECT * FROM HALLO", objVL.contains("SELECT * FROM HALLO"));
		objVL.AppendValue("SELECT * FROM HALLO1;SELECT * FROM HALLO2");
		assertTrue("contains SELECT * FROM HALLO", objVL.contains("SELECT * FROM HALLO"));
		System.out.println("JSOptionValueListTest.testAppendValue()"
				+ objVL.concatenatedValue(";"));
		assertTrue("contains SELECT * FROM HALLO1", objVL.contains("SELECT * FROM HALLO1"));
		assertTrue("contains SELECT * FROM HALLO2", objVL.contains("SELECT * FROM HALLO2"));
		assertTrue("contains SELECT * FROM HALLO", objVL.contains("SELECT * FROM HALLO"));

		objVL = new JSOptionValueList(objOptions, "ValueList", "Descr", "", false);
		objVL.AppendValue("SELECT * FROM HALLO1;SELECT * FROM HALLO2");
		assertTrue("contains SELECT * FROM HALLO1", objVL.contains("SELECT * FROM HALLO1"));
		assertTrue("contains SELECT * FROM HALLO2", objVL.contains("SELECT * FROM HALLO2"));
		objVL.AppendValue("HUHU");
		assertTrue("contains SELECT * FROM HALLO1", objVL.contains("HUHU"));

		
		// Erster Konstruktor
		objVL = new JSOptionValueList(objOptions, "ValueList", "Descr", "", "", false);
		objVL.AppendValue("SELECT * FROM HALLO1;SELECT * FROM HALLO2");
		assertTrue("contains SELECT * FROM HALLO1", objVL.contains("SELECT * FROM HALLO1"));
		assertTrue("contains SELECT * FROM HALLO2", objVL.contains("SELECT * FROM HALLO2"));

		
	}
	
	@Test
	public void testKonstruktor4IndexedKey() throws Exception {
		strStatements[0] = "SELECT * FROM HALLO;SELECT ABC FROM TABELLE1;";
		strStatements[1] = "SELECT * FROM TABELLE2;";
		settings.put(strKey, strStatements[0]);
		settings.put(strKey + "1", strStatements[1]);
		
		JSOptionValueList objVL = new JSOptionValueList(objOptions, strKey, "Descr", "", false);
		int intActualLength = objVL.value().length;
		int intExpectedLength = 2;
		assertEquals("Anzahl indexed Options ist korrekt", intExpectedLength, intActualLength);
		
		
		objVL.Value(objVL.concatenatedValue(strKey));
		intActualLength = objVL.value().length;
		intExpectedLength = 3;
		assertEquals("Anzahl indexed Options ist korrekt", intExpectedLength, intActualLength);
		
		
	}
	
	@Test
	@Ignore
	public void testValue() throws Exception {
		strStatements[0] = "SELECT * FROM HALLO;SELECT ABC FROM TABELLE1;";
		strStatements[1] = "SELECT * FROM TABELLE222;";
		strStatements[2] = "SELECT SPALTE1 FROM TABELLE5;";
		settings.put(strKey + "1", strStatements[0]);
		//	settings.put(strKey + "2", strStatements[1]);
		settings.put(strKey + "3", strStatements[2]);
		optionValueList = new JSOptionValueList(objOptions, strKey, strDescr + "7", "", true);
		assertEquals("Value failed", strStatements[0], optionValueList.Value());
		
	}
	
	
	
	@Test
	public void testContains() throws Exception {

		JSOptionValueList objVL = new JSOptionValueList(objOptions, "ValueList", "Descr", "Das|ist|das|Haus|vom|Nikolaus", "", false);

		assertTrue("contains Nikolaus", objVL.contains("Nikolaus"));
		assertFalse("contains Nikolaus", objVL.contains("kolaus"));

		objVL.Value("Das|ist|das|Haus|vom|Nikolaus");

		assertTrue("contains Nikolaus", objVL.contains("Nikolaus"));
		assertFalse("contains Nikolaus", objVL.contains("kolaus"));

	}

	@Test
	public void testElementAt() throws Exception {
		JSOptionValueList objVL = new JSOptionValueList(objOptions, "ValueList", "Descr", "Das|ist|das|Haus|vom|Nikolaus", "", false);

		assertEquals("Element 1", "Das", objVL.ElementAt(0));
		assertEquals("Element 2", "ist", objVL.ElementAt(1));
		assertEquals("Element 3", "das", objVL.ElementAt(2));
		assertEquals("Element 4", "Haus", objVL.ElementAt(3));
		assertEquals("Element 5", "vom", objVL.ElementAt(4));
		assertEquals("Element 6", "Nikolaus", objVL.ElementAt(5));

	}

}
