/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.JSHelper.io.Files;


import static org.junit.Assert.assertEquals;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;

import com.sos.JSHelper.Listener.JSListenerClass;
import com.sos.JSHelper.Options.JSOptionsClass;

/**
 * \class JSCsvFileTest
 *
 * \brief JSCsvFileTest -
 *
 * \details
 *
 * \section JSCsvFileTest.java_intro_sec Introduction
 *
 * \section JSCsvFileTest.java_samples Some Samples
 *
 * \code
 *   .... code goes here ...
 * \endcode
 *
 * <p style="text-align:center">
 * <br />---------------------------------------------------------------------------
 * <br /> APL/Software GmbH - Berlin
 * <br />##### generated by ClaviusXPress (http://www.sos-berlin.com) #########
 * <br />---------------------------------------------------------------------------
 * </p>
 * \author EQALS
 * \version 18.05.2009
 * \see reference
 *
 * Created on 18.05.2009 15:49:49
 */

public class JSCsvFileTest extends JSListenerClass { 

	@SuppressWarnings("unused")
	private final String	conClassName	= "JSCsvFileTest";
	private static JSFile fleTestdataDirectory = null;

	public JSCsvFileTest() {
		//
	}

	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		JSOptionsClass objOptionsClass = new JSOptionsClass();
		fleTestdataDirectory = new JSFile(objOptionsClass.UserDir() + "junit/testdata/");
	}

	@AfterClass
	public static void tearDownAfterClass() throws Exception {
	}

	@Before
	public void setUp() throws Exception {
	}

	@After
	public void tearDown() throws Exception {
	}

	/**
	 *
	 * \brief testReadCsvFile1
	 *
	 * \details
	 * Test CSV-File mit Kommentaren
	 *
	 * \return void
	 *
	 * @throws Exception
	 */
	@Test
	@Ignore
	public void testReadCsvFile1() throws Exception {
		String[] headers = null;
		String[] strValues = null;

		JSCsvFile objCsvFile = new JSCsvFile(fleTestdataDirectory + "/StoppageList.csv");
		objCsvFile.registerMessageListener(this);
		objCsvFile.loadHeaders();
		headers =  objCsvFile.Headers();
		assertEquals("Header1 must be Reporter", headers[0], "Reporter");
		assertEquals("Header2 must be StoppageType", headers[1], "StoppageType");
		assertEquals("Header3 must be FromDate", headers[2], "FromDate");
		assertEquals("Header4 must be ToDate", headers[3], "ToDate");

		strValues = objCsvFile.readCSVLine();
		assertEquals("Field1 must be ## StoppageType: S=Stop, R=Run", strValues[0], "## StoppageType: S=Stop, R=Run");

		strValues = objCsvFile.readCSVLine();
		assertEquals("Field1 must be ### Inbound: S=Stop, R=Run", strValues[0], "### Inbound");

		strValues = objCsvFile.readCSVLine();
		assertEquals("Field1 must be # P2R             ", strValues[0], "# P2R             ");

		strValues = objCsvFile.readCSVLine();
		assertEquals("Field1 must be 2001            ", strValues[0], "2001            ");
		assertEquals("Field2 must be S", strValues[1], "S");
		assertEquals("Field3 must be         13.02.2009 23:00:00", strValues[2], "        13.02.2009 23:00:00");
		assertEquals("Field4 must be 15.02.2009 22:00:00", strValues[3], "15.02.2009 22:00:00");
	}

	/**
	 *
	 * \brief testReadCsvFile2
	 *
	 * \details
	 * CSV-File ohne Kommentare
	 *
	 * \return void
	 *
	 * @throws Exception
	 */
	@Test
	public void testReadCsvFile2() throws Exception {
		String[] headers = null;
		String[] strValues = null;

		JSCsvFile objCsvFile = new JSCsvFile(fleTestdataDirectory + "/test.csv");
		objCsvFile.registerMessageListener(this);
		objCsvFile.loadHeaders();
		headers =  objCsvFile.Headers();
		assertEquals("Header1 must be Reporter", headers[0], "Reporter");
		assertEquals("Header2 must be StoppageType", headers[1], "StoppageType");
		assertEquals("Header3 must be FromDate", headers[2], "FromDate");
		assertEquals("Header4 must be ToDate", headers[3], "ToDate");

		strValues = objCsvFile.readCSVLine();
		assertEquals("Field1 must be 2001            ", strValues[0], "2001            ");
		assertEquals("Field2 must be S", strValues[1], "S");
		assertEquals("Field3 must be         13.02.2009 23:00:00", strValues[2], "        13.02.2009 23:00:00");
		assertEquals("Field4 must be 15.02.2009 22:00:00", strValues[3], "15.02.2009 22:00:00");

		strValues = objCsvFile.readCSVLine();
		assertEquals("Field1 must be 0032            ", strValues[0], "0032            ");
		assertEquals("Field2 must be S", strValues[1], "S");
		assertEquals("Field3 must be         13.02.2009 23:00:00", strValues[2], "        13.02.2009 23:00:00");
		assertEquals("Field4 must be 15.02.2009 22:00:00", strValues[3], "15.02.2009 22:00:00");

	}
}
