/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.JSHelper.Logging;

import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Layout;
import org.apache.log4j.Level;
import org.apache.log4j.spi.LoggingEvent;

import sos.util.SOSLogger;
import sos.util.SOSSchedulerLogger;

/**
* \class JobSchedulerLog4JAppender 
* 
* \brief JobSchedulerLog4JAppender - Redirect Log4J-Messages to SOSSchedulerLogger
* 
* \details
* This Class acts as a redirection for all Log4J-Messages to the JobScheduler Logger.
* 
* 
* \section JobSchedulerLog4JAppender.java_intro_sec Introduction
*
* \section JobSchedulerLog4JAppender.java_samples Some Samples
*
* \code
*   .... code goes here ...
* \endcode
*
* <p style="text-align:center">
* <br />---------------------------------------------------------------------------
* <br /> APL/Software GmbH - Berlin
* <br />##### generated by ClaviusXPress (http://www.sos-berlin.com) #########
* <br />---------------------------------------------------------------------------
* </p>
* \author KB
* \version 13.06.2010
* \see reference
*
* Created on 13.06.2010 12:08:56
 */

/**
 * @author KB
 *
 */
public class JobSchedulerLog4JAppender extends ConsoleAppender {

	private final String		conClassName	= "JobSchedulerLog4JAppender";

	private SOSSchedulerLogger	sosLogger		= null;

	public JobSchedulerLog4JAppender() {
		super();
	}

	public JobSchedulerLog4JAppender(Layout pobjLayout) {
		super(pobjLayout);
	}

	public void setSchedulerLogger(SOSSchedulerLogger pobjSchedulerLogger) {
		sosLogger = pobjSchedulerLogger;
	}

	/** 
	 * @see org.apache.log4j.FileAppender#activateOptions() */
	@Override
	public void activateOptions() {
		super.activateOptions();
	}

	/** 
	 * @see org.apache.log4j.WriterAppender#subAppend(LoggingEvent) */
	@Override
	protected void subAppend(LoggingEvent event) {
		// Nicht 'super.subAppend(event)' aufrufen!
		String strMsg = getLayout().format(event);
		if (null != sosLogger) {
			try {
				Level lL = event.getLevel();
				int intL = lL.toInt();
				int intSOSLevel = 0;
				switch (intL) {
					case Level.DEBUG_INT:
					case Level.TRACE_INT:
						intSOSLevel = SOSLogger.DEBUG;
						break;

					case Level.WARN_INT:
					case Level.INFO_INT:
						intSOSLevel = SOSLogger.INFO;
						break;

					case Level.ERROR_INT:
						intSOSLevel = SOSLogger.ERROR;
						break;

					case Level.FATAL_INT:
						intSOSLevel = SOSLogger.WARN;
						break;

					default:
						intSOSLevel = SOSLogger.INFO;
						break;
				}
//				strMsg = strMsg.replaceAll("\\n", "");
				if (strMsg.trim().length() > 0) {
					sosLogger.log(intSOSLevel, strMsg);
				}
			}
			catch (Exception ex) {
			}
		}
		else {
			System.out.println(conClassName + ": " + strMsg);
		}
	}

}
