/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package sos.net.ssh;

import org.apache.log4j.Logger;

import com.sos.JSHelper.Basics.JSJobUtilities;
import com.sos.JSHelper.Basics.JSToolBox;
import com.sos.VFS.Factory.VFSFactory;
import com.sos.VFS.Interfaces.ISOSVFSHandler;

/**
 * \class SOSSSHJob2 - Start a Script or command using SSH
 * 
 * \brief Start a Script using SSH
 * 
 * Start a Script using SSH
 * 
 * \section
 * 
 * \see
 * 
 * \code .... 
 * code goes here ... 
 * \endcode
 * 
 * \author Klaus Buettner - http://www.sos-berlin.com \version 1.1.0.20100506
 * 
 * This Source-Code was created by JETTemplate SimpleClass.javajet, Version 1.0
 * from 2009-12-26, written by kb
 */

public class SOSSSHJob2 extends JSToolBox implements JSJobUtilities {

	private final String		conClassName		= "SOSSSHJob2";

	private Logger				logger				= Logger.getLogger(SOSSSHJob2.class);

	/** array of commands that have been separated by the commandDelimiter */
	protected String[]			strCommands2Execute	= {};

	/** Output from stdout and stderr **/
	protected StringBuffer		strbStdoutOutput;
	protected StringBuffer		strbStderrOutput;

	private ISOSVFSHandler		objVFS				= null;

	private JSJobUtilities		objJSJobUtilities	= this;

	protected SOSSSHJobOptions	objOptions			= null;

	/**
	 * 
	 * \brief getVFS
	 * 
	 * \details
	 *
	 * \return ISOSVFSHandler
	 *
	 * @return
	 * @throws Exception
	 */
	private ISOSVFSHandler getVFS() throws Exception {

		// TODO type of connection/file-system as an option in Option-class

		if (objVFS == null) {
			objVFS = VFSFactory.getHandler("SSH2");
		}
		return objVFS;
	}

	/**
	 * 
	 * \brief SOSSSHJob2
	 *
	 * \details
	 *
	 */
	public SOSSSHJob2() {
		super();
		try {
			getVFS();
		}
		catch (Exception e) {
			throw new RuntimeException("could not initialize VFS", e);
		}
	}

	/**
	 * 
	 * \brief Options - returns the SOSSSHOptionClass
	 * 
	 * \details
	 * The SOSSSHOptionClass is used as a Container for all Options (Settings) which are
	 * needed to define the SSH-Connection and to specify what has to do on the SSH-Server.
	 *  
	 * \return SOSSSHJobOptions
	 *
	 * @return
	 */
	public SOSSSHJobOptions Options() {

		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::Options";

		if (objOptions == null) {
			objOptions = new SOSSSHJobOptions();
		}
		return objOptions;
	}

	/**
	 * 
	 * \brief setJSJobUtilites
	 * 
	 * \details
	 * The JobUtilities are a set of methods used by the SSH-Job or can be used be other, similar, job-
	 * implementations.
	 * 
	 * \return void
	 *
	 * @param pobjJSJobUtilities
	 */
	public void setJSJobUtilites(JSJobUtilities pobjJSJobUtilities) {
		
		if (pobjJSJobUtilities == null) {
			objJSJobUtilities = this;
		}
		else {
			objJSJobUtilities = pobjJSJobUtilities;
			objVFS.setJSJobUtilites(pobjJSJobUtilities);
		}
		logger.debug("objJSJobUtilities = " + objJSJobUtilities.getClass().getName());
	}

	/**
	 * 
	 * \brief Execute - Start the Execution of commands or scripts on the SSH-Server-Site
	 * 
	 * \details
	 * This Method is to start commands or a script on the SSH-Server-Site.
	 * What to start and how to handle is specified in the related Option-Class SOSSSHJobOptions.
	 * 
	 * For more details see
	 * 
	 * \see SOSSSHJob2JSAdapter 
	 * \see SOSSSH2Main
	 * 
	 * \return SOSSSHJob2
	 *
	 * @return
	 */
	public SOSSSHJob2 Execute() throws Exception {
		final String conMethodName = conClassName + "::Execute";

		logger.debug(conMethodName + " start ...");

		getVFS();
		objVFS.setJSJobUtilites(objJSJobUtilities);

		try {
			Options().CheckMandatory();
			logger.debug(Options().toString());

			objVFS.Connect(objOptions);
			objVFS.Authenticate(objOptions);
			objVFS.OpenSession(objOptions);
			objVFS.remoteIsWindowsShell();

			if (objOptions.command.IsEmpty() == false) {
				strCommands2Execute = objOptions.command.values();
			}
			else
				if (objOptions.isScript()) {
					strCommands2Execute = new String[1];
					String strTemp = objOptions.command_script.Value();
					if (objOptions.command_script.IsEmpty()) {
						strTemp = objOptions.command_script_file.JSFile().File2String();
						// 
						strCommands2Execute[0]  = objVFS.createScriptFile(strTemp);
					}
					else {
						strCommands2Execute[0] = objVFS.createScriptFile(strTemp);						
					}
					strCommands2Execute[0] += " " + objOptions.command_script_param.Value();
				}
				else {
					throw new RuntimeException("neither Commands nor Script(file) specified. Abort.");
				}

			for (String strCmd : strCommands2Execute) {
				try {
					logger.debug("executing remote command: " + strCmd);
					objVFS.ExecuteCommand(strCmd);
					strbStderrOutput = objVFS.getStdErrBuffer();
					if (isNotEmpty(strbStderrOutput)) {
						if (objOptions.ignore_stderr.value() == true) {
							logger.info("output to stderr is ignored: " + strbStderrOutput);
						}
						else {
							throw new RuntimeException("remote execution reports error: " + strbStderrOutput);
						}
					}

					Integer exitStatus = objVFS.getExitCode();
					if (exitStatus != null) {
						if (!exitStatus.equals(new Integer(0))) {
							if (objOptions.ignore_error.isTrue() || objOptions.ignore_exit_code.Values().contains(exitStatus)) {
								logger.info("exit code is ignored due to option-settings: " + exitStatus);
							}
							else {
								throw new RuntimeException(conMethodName + ": " + "remote command terminated with exit code: " + exitStatus);
							}
						}
					}

					String exitSignal = objVFS.getExitSignal();
					if (isNotEmpty(exitSignal)) {
						if (objOptions.ignore_signal.isTrue()) {
							logger.info("exit signal is ignored due to option-settings: " + exitSignal);
						}
						else {
							throw new RuntimeException(conMethodName + ": " + "remote command terminated with exit signal: " + exitSignal);
						}
					}
				}
				catch (Exception e) {
					throw e;
				}
				finally {
					objVFS.CloseConnection();
				}
			}
		}
		catch (Exception e) {
			logger.error(this.StackTrace2String(e));
			logger.error(conMethodName + ": " + "error occurred processing ssh command: ", e);
			throw e;
		}
		finally {
			objVFS.CloseConnection();
		}
		return this;
	}

	/**
	 * 
	 * \brief getStdOut - return the STDOUT from the SSH-Server
	 * 
	 * \details
	 * The STDOUT-messages as an result of the execution of the command.
	 * 
	 * \return StringBuffer
	 *
	 * @throws Exception
	 */
	public StringBuffer getStdOut() throws Exception {

		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::getStdOut";

		return this.getVFS().getStdOutBuffer();
	} // private StringBuffer getStdOut

	/**
	 * 
	 * \brief getStdErr
	 * 
	 * \details
	 *
	 * \return StringBuffer
	 *
	 * @return
	 * @throws Exception
	 */
	public StringBuffer getStdErr() throws Exception {

		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::getStdErr";

		return this.getVFS().getStdErrBuffer();
	} // private StringBuffer getStdOut

	public String myReplaceAll(String pstrSourceString, String pstrReplaceWhat, String pstrReplaceWith) {

		String newReplacement = pstrReplaceWith.replaceAll("\\$", "\\\\\\$");
		return pstrSourceString.replaceAll("(?m)" + pstrReplaceWhat, newReplacement);
	}

	/**
	 * 
	 * \brief replaceSchedulerVars
	 * 
	 * \details
	 * Dummy-Method to make sure, that there is always a valid Instance for the JSJobUtilities.
	 * \return 
	 *
	 * @param isWindows
	 * @param pstrString2Modify
	 * @return
	 */
	@Override
	public String replaceSchedulerVars(boolean isWindows, String pstrString2Modify) {
		logger.debug("replaceSchedulerVars as Dummy-call executed. No Instance of JobUtilites specified.");
		return pstrString2Modify;
	}

	/**
	 * 
	 * \brief setJSParam
	 * 
	 * \details
	 * Dummy-Method to make shure, that there is always a valid Instance for the JSJobUtilities.
	 * \return 
	 *
	 * @param pstrKey
	 * @param pstrValue
	 */
	@Override
	public void setJSParam(String pstrKey, String pstrValue) {
		
	}


} // SOSSSHJob2
