/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: Job_chain_node.java 4958 2007-04-08 18:31:41Z jz $

package sos.spooler;

/*+
 * Ein Jobkettenknoten beschreibt eine Stelle in einer Jobkette ({@link Job_chain}). 
 * Einem Jobkettenknoten sind zugeordnet: ein Zustand, ein Job, ein Folgezustand und ein Fehlerzustand.
 * <p>
 * Ein Jobkettenknoten wird mit {@link Job_chain#add_job(String,String,String,String)} oder mit {@link Job_chain#add_end_state(String)}
 * erzeugt.    
 *
 * @see Job_chain#node(String)
 * @see Order#job_chain_node()
 */
/** 
 * @author Joacim Zschimmer
 * @version $Revision: 4958 $
 */

public class Job_chain_node extends Idispatch
{
    private                 Job_chain_node      ( long idispatch )                  { super(idispatch); }


  //public void         set_name                ( String value )                    {                        com_call( ">name", value ); }


    
    /*+ Zustand, fr den dieser Jobkettenknoten gilt.
     * 
     * <p><br/><b>Beispiel</b>
     * <pre>
     *     Job_chain_node job_chain_node = spooler.job_chain( "Jobkette" ).node( 100 );                                                  
     *     spooler_log.debug( "state=" + job_chain_node.state() );                      // "state=100"
     * </pre>
     *
     * <p><br/><b>Beispiel in JavaScript</b>
     * <pre>
     *     var job_chain_node = spooler.job_chain( "Jobkette" ).node( 100 );                                                  
     *     spooler_log.debug( "state=" + job_chain_node.state );                        // "state=100"
     * </pre>
     */

    public String           state               ()                                  { return (String)        com_call( "<state"       ); }


    
    /*+ Liefert den nchsten Knoten in der Jobkette fr den Erfolgsfall.
     * 
     * <p><br/><b>Beispiel</b>
     * <pre>
     *     Job_chain_node job_chain_node = spooler.job_chain( "Jobkette" ).node( 100 );                                                  
     *     spooler_log.debug( "next state=" + job_chain_node.next_node().state() );     // "state=200"
     * </pre>
     *
     * <p><br/><b>Beispiel in JavaScript</b>
     * <pre>
     *     var job_chain_node = spooler.job_chain( "Jobkette" ).node( 100 );                                                  
     *     spooler_log.debug( "next state=" + job_chain_node.next_node.state );         // "state=200"
     * </pre>
     *  
     * @return null, wenn es keinen nchsten Knoten gibt (Folgezustand ist nicht angegeben) 
     */
    public Job_chain_node   next_node           ()                                  { return (Job_chain_node)com_call( "<next_node"   ); }


    
    /*+ Liefert den nchsten Knoten in der Jobkette fr den Fehlerfall.
     *  
     * <p><br/><b>Beispiel</b>
     * <pre>
     *     Job_chain_node job_chain_node = spooler.job_chain( "Jobkette" ).node( 100 );                                                  
     *     spooler_log.debug( "error state=" + job_chain_node.error_node().state() );   // "state=999"
     * </pre>
     *
     * <p><br/><b>Beispiel in JavaScript</b>
     * <pre>
     *     var job_chain_node = spooler.job_chain( "Jobkette" ).node( 100 );                                                  
     *     spooler_log.debug( "error state=" + job_chain_node.error_node.state );       // "state=999"
     * </pre>
     * 
     * @return null, wenn es keinen Fehler-Knoten gibt (Fehlerzustand ist nicht angegeben) 
     */
    public Job_chain_node   error_node          ()                                  { return (Job_chain_node)com_call( "<error_node"  ); }


    /*+ Liefert den Job, der dem Knoten zugeordnet ist.
     * 
     * <p><br/><b>Beispiel</b>
     * <pre>
     *     Job_chain_node job_chain_node = spooler.job_chain( "Jobkette" ).node( 100 );                                                  
     *     spooler_log.debug( "job=" + job_chain_node.job().name() );                   // "job=job_100"
     * </pre>
     *
     * <p><br/><b>Beispiel in JavaScript</b>
     * <pre>
     *     var job_chain_node = spooler.job_chain( "Jobkette" ).node( 100 );                                                  
     *     spooler_log.debug( "job=" + job_chain_node.job.name );                       // "job=job_100"
     * </pre>
     */
    public Job              job                 ()                                  { return (Job)           com_call( "<job"         ); }


    /*+ Liefert den nchsten Zustand in der Jobkette fr den Erfolgsfall.
     * 
     * <p><br/><b>Beispiel</b>
     * <pre>
     *     Job_chain_node job_chain_node = spooler.job_chain( "Jobkette" ).node( 100 );                                                  
     *     spooler_log.debug( "next state=" + job_chain_node.next_state() );            // "state=200"
     * </pre>
     *
     * <p><br/><b>Beispiel in JavaScript</b>
     * <pre>
     *     var job_chain_node = spooler.job_chain( "Jobkette" ).node( 100 );                                                  
     *     spooler_log.debug( "next_state=" + job_chain_node.next_state );              // "state=200"
     * </pre>
     */
    public String           next_state          ()                                  { return (String)        com_call( "<next_state"  ); }


    /*+ Liefert den nchsten Zustand in der Jobkette fr den Fehlerfall.
     *  
     * <p><br/><b>Beispiel</b>
     * <pre>
     *     Job_chain_node job_chain_node = spooler.job_chain( "Jobkette" ).node( 100 );                                                  
     *     spooler_log.debug( "error state=" + job_chain_node.error_node().state() );   // "error state=999"
     * </pre>
     *
     * <p><br/><b>Beispiel in JavaScript</b>
     * <pre>
     *     var job_chain_node = spooler.job_chain( "Jobkette" ).node( 100 );                                                  
     *     spooler_log.debug( "error state=" + job_chain_node.error_node.state );       // "error state=999"
     * </pre>
     *  
     */
    public String           error_state         ()                                  { return (String)        com_call( "<error_state" ); }


    public final static String ACTION_PROCESS    = "process";
    public final static String ACTION_STOP       = "stop";
    public final static String ACTION_NEXT_STATE = "next_state";
    
    public void             set_action          ( String action )                   {                        com_call( ">action", action ); }
    public String           action              ()                                  { return (String)        com_call( "<action"         ); }
}
