/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/

package sos.stresstest;

import java.io.InputStream;
import java.lang.reflect.*;
import java.net.HttpURLConnection;
import java.net.URL;

import sos.connection.SOSConnection;
import sos.settings.SOSProfileSettings;
import sos.stresstest.Stresstest;
import sos.util.SOSLogger;

/**
 * @author Joacim Zschimmer <j@zsch.de>
 * @since 2004-08-10
 * @version 1.0
 * @author Andreas Pschel <andreas.pueschel@sos-berlin.com>
 * @since 2004-11-21
 * @version 1.1
 * 
 * 
 * StresstestSession kann als abstrakte Klasse nicht instanziiert werden, jedoch kann der StresstestThread
 * von ihr erbende nicht abstrakte Klassen instanziieren. Dadurch ist es mglich nachtrglich 
 * Java-Klassen zu schreiben auf die der StresstestThread ber den Klassennamen zugreifen kann
 *  ohne das im StresstestThread, StresstestSession und SOSStresstestSession verndert werden muss.
 * 
 */

public abstract class StresstestSession
{
    private final StresstestThread  _stresstest_thread;
    private final int               _session_number;
    private final Stresstest		_stresstest  = null; 
    private       String            _action_name = new String("");
    //----------------------------------------------------------------------------StresstestSession
    
    public StresstestSession( StresstestThread stresstest_thread )
    {
        _stresstest_thread = stresstest_thread;
        _session_number    = stresstest_thread._current_session_number;
    }
    
    //-------------------------------------------------------------------------------------init
    
    /**
     * Initialisierung
     */
    public void init    () throws Exception {};

    //-------------------------------------------------------------------------------------destruct
    
    /**
     * Destructor
     */
    public abstract void destruct    () throws Exception;

    //-------------------------------------------------------------------------------get_stresstest
    
	/**
	 * @return Returns the _stresstest.
	 */
    protected Stresstest get_stresstest() {
		return _stresstest_thread._stresstest;
	}
	
    //---------------------------------------------------------------------------------executeChain
    
	/**
     *   
     * Fhrt eine Aktionskette aus. muss in allen Klassen, die von StresstestSession erben,
     * aber nicht abstrakt sind, implementiert werden
     */
    public abstract void executeChain() throws Exception;
	
    //--------------------------------------------------------------------------------getConnection
    
    /**
     * Gibt die aktuelle Verbindung zur Datenbank zurck
     */
    public SOSConnection getConnection() { return _stresstest_thread.getConnection(); }
    
    //-------------------------------------------------------------------------------get_connection
    
    /**
     * Gibt die aktuelle Verbindung zur Datenbank zurck
     */
    public SOSConnection get_connection() { return _stresstest_thread.get_connection(); }
    

    //------------------------------------------------------------------------------------getLogger
    
    /**
     * Gibt den aktuellen Logger zurck
     */
    public SOSLogger getLogger() { return _stresstest_thread.getLogger(); }
    
    //-----------------------------------------------------------------------------------get_logger
    
    /**
     * Gibt den aktuellen Logger zurck
     */
    public SOSLogger get_logger() { return _stresstest_thread.get_logger(); }
    

    //-------------------------------------------------------------------------------get_settings
    
    /**
     * Gibt das Objekt der Einstellungen zurck
     */
    public SOSProfileSettings get_settings() { return _stresstest_thread.get_settings(); }
    
    //-------------------------------------------------------------------------get_settings_entry
    
    /**
     * Gibt den Wert einer Einstellungen zurck
     */
    public String get_settings_entry(String name) throws Exception {
    	return _stresstest_thread.get_settings().getSectionEntry(_action_name, name); 
    }
    
    //-------------------------------------------------------------------------get_settings_entry
    
    /**
     * Gibt den Wert einer Einstellungen zurck
     */
    public String get_settings_entry(String section, String name) throws Exception {
    	return _stresstest_thread.get_settings().getSectionEntry(section, name); 
    }
    
    //---------------------------------------------------------------------------get_session_number
    
    /**
     * Gibt die aktuelle Session Nummer zurck
     */
    public int get_session_number() { return _session_number; }
    
    /** Gibt den Index des Threads zurck
	 * @return Returns the _thread_index.
	 */
    
    //-----------------------------------------------------------------------------get_thread_index
    
	public int get_thread_index() {
		return _stresstest_thread.get_thread_index();
	}
    //--------------------------------------------------------------------------------------execute
    
    /**
     * Startet die Protokollierung und die Abarbeitung der SQL-Anweisungen von einer nicht abstrakten 
     * von StresstestSession erbenden Klasse
     * @param action_name Name der Funktion die von der instanziierten Klasse ausgefhrt werden soll
     * @throws Exception 
     */  
    
    public StresstestResponse execute( String action_name ) throws Exception
    {
    	_action_name = action_name;
    	StresstestResponse response = new StresstestResponse();
    	
    	if (get_stresstest().get_execute() != null && get_stresstest().get_execute().length() > 0 
        && action_name != null && action_name.length() > 0) {
    		if (get_stresstest().get_execute().toLowerCase().indexOf(action_name.toLowerCase()) == -1) { return response; }
    	}
    	
        StresstestThread.Time_record  time_record = _stresstest_thread.new Time_record( _session_number,action_name);
        String                        method_name = action_name;
        Method method = getClass().getMethod( method_name, null );
        try
        {
        	String[] arguments =null;
            method.invoke( this, arguments);
            response.setResponseCode(0);
            return response;
        }
        catch( InvocationTargetException x )
        {
        	response.setResponseMessage(x.getMessage());
            time_record._throwable = x.getCause();
            throw x;
        }
        finally
        {
            _stresstest_thread.add_time_record( time_record );
        }
    }
    	
    /**
     * Startet die Protokollierung und die Abarbeitung der SQL-Anweisungen von einer nicht abstrakten
     * von StresstestSession erbenden Klasse (standardmssig SOSStresstestSession)
     * @param action_name Name der auszufhrenden Funktion
     * @param filename Dateiname der Eingabedatei
     * @throws Exception
     */
    public StresstestResponse execute( String action_name, String filename ) throws Exception
    {
    	_action_name = action_name;
    	StresstestResponse response = new StresstestResponse();
    	
    	if (get_stresstest().get_execute() != null && get_stresstest().get_execute().length() > 0 
    	        && action_name != null && action_name.length() > 0) {
    		if (get_stresstest().get_execute().toLowerCase().indexOf(action_name.toLowerCase()) == -1) { return response; }
    	}
    	
        StresstestThread.Time_record  time_record = _stresstest_thread.new Time_record( _session_number, filename);
        String                        method_name = action_name;
        Method method = getClass().getMethod( method_name, null );
        try
        {
            method.invoke( this, null);
            response.setResponseCode(0);
            return response;
        }
        catch( InvocationTargetException x )
        {
        	response.setResponseMessage(x.getMessage());
            time_record._throwable = x.getCause();
            throw x;
        }
        finally
        {
            _stresstest_thread.add_time_record( time_record );
        }
    }

    /**
     * Startet die Protokollierung und die Abarbeitung der URLs von einer nicht abstrakten
     * von StresstestSession erbenden Klasse (standardmssig SOSStresstestSession)
     * @param url auszufhrende URL
     * @throws Exception
     */
    public StresstestResponse execute( URL url) throws Exception {
    	
    	_action_name = url.toString();
    	StresstestResponse response = new StresstestResponse();
    	
    	if (get_stresstest().get_execute() != null && get_stresstest().get_execute().length() > 0 
    	        && url.toString() != null && url.toString().length() > 0) {
    		if (get_stresstest().get_execute().toLowerCase().indexOf(url.toString().toLowerCase()) == -1) { return response; }
    	}
    	
        StresstestThread.Time_record  time_record = _stresstest_thread.new Time_record( _session_number, url.toString());

		HttpURLConnection connection = null;
		
        try {
    		connection = (HttpURLConnection)url.openConnection();
    		connection.connect();
    			
    		InputStream in = connection.getInputStream();
    		response.setResponseCode(connection.getResponseCode());
    		response.setResponseMessage(connection.getResponseMessage());
    			
    		if (response.getResponseCode() >= 200 && response.getResponseCode() < 300) {  
    			byte buffer[] = new byte[1024];
    			int bytesRead;
    			while ( (bytesRead = in.read(buffer)) != -1) {
    				// System.out.write(buffer, 0, bytesRead);
    			}
    		}
    			
    		connection.disconnect();
    		return response;
    			
    	} catch (Exception x) {
            time_record._throwable = x.getCause();
            throw x;
    	}
        finally
	    {
	        _stresstest_thread.add_time_record( time_record );
	    }
    }
    
}
