/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
/*
 * Created on 09.12.2004
 *  
 */
package sos.stresstest.dialogtest;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;

import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.UnknownHostException;

import java.net.URL;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import sos.util.SOSClassUtil;
import sos.util.SOSLogger;

/**
 * @author ur
 * 
 * Ein Dialogschritt innerhalb eines zu testenden Checkpoints
 */

public class DialogStep extends DialogTesterBasic {
		
    private URL url = null;
    private int nr = 0;
    //private HashMap dialogstep;
    private HashMap request;
    private HashMap header;
    private DialogTester caller;
    private DialogCheckpoint checkpoint;
    private String requestMethod;
    
//  response time des Servers fr diesen Request
		private long responseTime = 1;
		
		// die Dauer des gesamten Steps
		private long duration = 0;
    
    protected DialogStep(SOSLogger sosLogger_, String u, int n, HashMap r, HashMap h, HashMap d)
            throws Exception {
				
        url = new URL(u);
       	
        nr = n;
        r.put("skip_recording", "1"); // Damit die Ausfhrung des Mitschnitts
                                      // nicht mitgeschnitten wird.
        request = r;
        header = h;
        //dialogstep = d;
        sosLogger = sosLogger_;
    }
		
    protected void printHeader() throws Exception {
    	
    	try {
        log_info("Header");
        log_info("-------");
        Iterator keys = header.keySet().iterator();
        Iterator vals = header.values().iterator();
				while (keys.hasNext()) {
					log_info(keys.next() + "=" + vals.next());
        }
    	} catch (Exception e) {
    		String err_msg = "";
  			if (this.getDebugLevel()>2)
  				err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
    		throw new Exception(err_msg + e.getMessage());
    	}
    }
		
    protected void printRequest() throws Exception {
    	
    	try {
        log_info("Request");
        log_info("-------");
        Iterator keys = request.keySet().iterator();
        Iterator vals = request.values().iterator();
        while (keys.hasNext()) {
            log_info(keys.next() + "=" + vals.next());
        }
    	} catch (Exception e) {
    		String err_msg = "";
  			if (this.getDebugLevel()>2)
  				err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
    		throw new Exception(err_msg + e.getMessage());
    	}
    }

    protected int getNr() {
        return nr;
    }

    protected URL getUrl() {
        return url;
    }
		
    protected void setUrl(URL _url) {
      this.url = _url;
  	}
    
		protected void showHeaderFields(HttpURLConnection connection) throws Exception {
    	
			try {
	    	Map fields = connection.getHeaderFields();
	  		Iterator it = fields.keySet().iterator();
				
	  		this.log_info("Header-Fields: ____________________________________________________");
	  		
	  		while(it.hasNext()) {
	  			String key = (String)it.next();
	  			String value = fields.get(key).toString();
	  			
	  			this.log_info("Header-Field:  key=" + key + "  value=" + value);
	  		}
	  		this.log_info("___________________________________________________________________");
			} catch (Exception e) {
				String err_msg = "";
				if (this.getDebugLevel()>2)
					err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
	  		throw new Exception(err_msg + e.getMessage());
			}
		}
		
		/**
		 * ermittelt die SessionID innerhalb eines Headers.
		 * @param fields der Header als Map
		 * @param subKey der Schlssel (der sessionIDKey)
		 * @return die sessionID
		 */
		private String identifySessionID(Map fields, String sessionIDKey) throws Exception {
			
			String sessionID = null;
			
			try {
				
				if ( sessionIDKey==null ) {
					this.log_debug4("DialogStep.identifySessionID: sessionIDKey is null");
					return null;
				}
				
				Iterator it = fields.keySet().iterator();
				
				Pattern pattern = Pattern.compile( ".*"+sessionIDKey+"=([\\dA-Fa-f]+).*");
				Matcher matcher;
				String key;
				String value;
				
				while( it.hasNext() && (sessionID==null) ) {
					
					key = (String)it.next();
					value = fields.get(key).toString();
					matcher = pattern.matcher( value );
					
					if ( matcher.matches() ) {
						if (matcher.groupCount() == 1) {
							sessionID = matcher.group(1);
						}
					}
				}
				
			} catch (Exception e) {
				String err_msg = "";
				if (this.getDebugLevel()>2)
					err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
	  		throw new Exception(err_msg + e.getMessage());
			}
			
			return sessionID;
		}

		// Den Header setzen.
		private HttpURLConnection setHeader(HttpURLConnection httpURLCon, String oldSessionID) throws Exception {
			
			String k, v = "";
			Pattern p;
      Matcher m;
      String oldV;
			
      try {
      	
	      Iterator headerVals = header.values().iterator();
	      Iterator headerKeys = header.keySet().iterator();
	      log_debug4("Execute Step:" + nr + " --> Set Header");
				
	      while (headerKeys.hasNext()) {
	      		
	          k = headerKeys.next().toString();
	          v = headerVals.next().toString();
						
	          if ( (caller.getSessionIDKey()!=null) && (caller.getSessionID()!=null) && (oldSessionID!=null) ) {
	          	
	          	p = Pattern.compile( oldSessionID );
	          	m = p.matcher( v );
	          	
	          	oldV = v;
	          	v = m.replaceAll( caller.getSessionID() );
	          	
	          	if ( !v.equals(oldV) ) {
	          		caller.setSessionIDReplaces(true);
	          		this.log_debug4("Cookie-Replacement:");
	          		this.log_debug4("old: "+oldV);
	          		this.log_debug4("new: "+v);
	          	}
	          }
						
	          httpURLCon.setRequestProperty(k, v);
	          log_debug4("Execute Step:" + " " + k + "=" + v);
	      }
				
			} catch (Exception e) {
				String err_msg = "";
				if (this.getDebugLevel()>2)
					err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
	  		throw new Exception(err_msg + e.getMessage());
			}
			
			return httpURLCon;
		}
		
		private void writeRequest(HttpURLConnection conn) throws Exception {
			
			String k, v = "";
			
			try {
				
				log_debug5("HttpURLConnection: "+conn.toString());
				
				if ( conn.getErrorStream() != null) {
	        InputStreamReader isr = new InputStreamReader(conn.getErrorStream());
	    		BufferedReader br = new BufferedReader(isr);
	    		String line = null;
	    		while ( (line = br.readLine()) != null) {
	    			this.log_debug5(line);
	    		}            
				} else {
					//log_debug5("ErrorStream is null");
				}
				
				OutputStreamWriter osw;
				
				if ( conn.getOutputStream() != null) {
					osw = new OutputStreamWriter(conn.getOutputStream());
				} else {
					throw new Exception("OutputStream is null");
				}
				
	      Iterator requestVals = request.values().iterator();
	      Iterator requestKeys = request.keySet().iterator();
	      String und = "";
	      
	      log_debug4("Execute Step:" + nr + " --> Set Request");
	      while (requestKeys.hasNext()) {
	          k = requestKeys.next().toString();
	          v = requestVals.next().toString();
						
	          osw.write(und + k + "=" + v);
	          und = "&";
	          log_debug4("Execute Step:" + " " + k + "=" + v);
	      }
				
	      osw.flush();
	      osw.close();
				
			} catch (Exception e) {
				String err_msg = "";
				if (this.getDebugLevel()>2)
					err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
	  		throw new Exception(err_msg + e.getMessage());
			}
		}
		
		private void processResponse(HttpURLConnection httpURLCon) throws Exception {
			
			DialogTesterChecks check = null;
			InputStream is = null;
			InputStreamReader isr = null;
			BufferedReader in = null;
			String buffer = null;
			File f = null;
			BufferedWriter outputFile = null;
			
			try {
				
			// response einlesen
				in = new BufferedReader( isr = new InputStreamReader( is = httpURLCon.getInputStream() ) );
				
				if ( in.ready() ) {
					buffer = "";
					for ( String line; (line = in.readLine()) != null; )
						buffer += line+'\n';
				}
				in.close();
				isr.close();
				is.close();
				
			// response time ermitteln - "Zeit stoppen"
				this.responseTime = System.currentTimeMillis() - this.responseTime;
				
			// HTML im Protokoll anzeigen
				if ( caller.isShowHtml() ) {
					String show = "--- showHTML -------  Recording: "+caller.getNr()+",  Take: '"+checkpoint.getTake()+"',  Checkpoint: '"+checkpoint.getName()+"',  Step: "+this.nr+"  -----------------------\n";
					show += buffer;
					show += " --------------------------- end showHTML ------------------------------------------------------------------------------------------------------";
					this.log_info(show);
				}
				
			// HTML wird in eine Datei geschrieben
				if ( caller.isCopyHtmlToFile() ) {
					
					f = new File(caller.getPath() + checkpoint.getTake() + "_" + checkpoint.getName() + "_" + nr + ".html");
					f.delete();		// alte Datei lschen
					outputFile = new BufferedWriter( new FileWriter(f, true) );
					outputFile.write(buffer);
					outputFile.flush();
					outputFile.close();
				}
				
			// HTML-Checks und Werte von Variablen extrahieren
				if ( buffer!=null && !buffer.equals("") ) {
					
					for (int i = 0; i < checkpoint.getListOfChecks().size(); i++) {
						
						check = (DialogTesterChecks) checkpoint.getListOfChecks().get(i);
						
						FilterMatcher fm = new FilterMatcher(check.getSearchString(), buffer, caller.getGlobalVars(), Pattern.CASE_INSENSITIVE);
						fm.setDebugLevel(this.getDebugLevel());
						
						// ausgewerteter String mit Wildcards an check selbst liefern
						check.setEvalVarString( fm.getSearchString() );
						
						log_debug4("extended matching with Vars: " + check.getSearchString());
						log_debug3("...matching \""+ fm.getSearchString() +"\"");
						
	          if ( !fm.find() ) {
	              log_debug3("Step:" + nr + " --> " + fm.getSearchString()
	                      + " not contained in HTML [not found]");
	          } else {
	              log_debug3("Step:" + nr + " --> " + fm.getSearchString() + " found");
	              log_debug4("show set vars: "+fm.showVars());
	              check.setFound(true);
	              // Wenn mindestens ein Step den Suchstring enthlt, wird "true" geliefert
						}
					}
				}
				
			} catch (Exception e) {
				String err_msg = "";
				if (this.getDebugLevel()>2)
					err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
	  		throw new Exception(err_msg + e.getMessage());
			}
		}
		
/*
		private void processResponseOLD(HttpURLConnection httpURLCon) throws Exception {
			
			DialogTesterChecks check;
			StringBuffer buffer = null;
			File f = null;
			BufferedWriter outputFile = null;
			InputStream inputStream = null;
			
			try {
				
				inputStream = httpURLCon.getInputStream();
				buffer = new StringBuffer();
				
				// html-Seite des Steps lschen, falls sie schon existiert
				if (caller.isCopyHtmlToFile()) {
					f = new File(caller.getPath() + checkpoint.getTake() + "_" + checkpoint.getName() + "_" + nr + ".html");
					f.delete();
				}
				
				
				if (caller.isCopyHtmlToFile() || caller.isShowHtml() || checkpoint.getListOfChecks().size() > 0) {
					
						if (caller.isCopyHtmlToFile()) {
							outputFile = new BufferedWriter( new FileWriter(f, true) );
						}
						
						String line = "";
						int a = 0;
						
						while (a != -1) {
							
							a = inputStream.read();
							
							if ( caller.isShowHtml() ) {
								if ( ((char)a)=='\n' ) {
									this.log_info("output: "+line);
									line = "";
								} else
									line += (char)a;
							}
							
							if (caller.isCopyHtmlToFile() || checkpoint.getListOfChecks().size() > 0) {
								buffer.append((char) a);
							}
						}
						
						// response time ermitteln
						this.responseTime = System.currentTimeMillis() - this.responseTime;
						
						if (caller.isCopyHtmlToFile()) {
							
							outputFile.write(buffer.toString());
							outputFile.flush();
							outputFile.close();
						}
						
	          inputStream.close();
	      }
				
				// Werte von Variablen extrahieren
				if ( buffer!=null && !buffer.equals("") ) {
					
					for (int i = 0; i < checkpoint.getListOfChecks().size(); i++) {
						
						check = (DialogTesterChecks) checkpoint.getListOfChecks().get(i);
						
						FilterMatcher fm = new FilterMatcher(check.getSearchString(), buffer.toString(), caller.getGlobalVars(), Pattern.CASE_INSENSITIVE);
						fm.setDebugLevel(this.getDebugLevel());
						
						// ausgewerteter String mit Wildcards an check selbst liefern
						check.setEvalVarString( fm.getSearchString() );
						
						log_debug4("extended matching with Vars: " + check.getSearchString());
						log_debug3("...matching \""+ fm.getSearchString() +"\"");
						
	          if ( !fm.find() ) {
	              log_debug3("Step:" + nr + " --> " + fm.getSearchString()
	                      + " not contained in HTML [not found]");
	          } else {
	              log_debug3("Step:" + nr + " --> " + fm.getSearchString() + " found");
	              log_debug4("show set vars: "+fm.showVars());
	              check.setFound(true);
	              // Wenn mindestens ein Step den Suchstring enthlt, wird "true" geliefert
						}
					}
				}
				
			} catch (Exception e) {
				String err_msg = "";
				if (this.getDebugLevel()>2)
					err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
	  		throw new Exception(err_msg + e.getMessage());
			}
		}
*/
		
		protected boolean execute() throws Exception {
			
			boolean success = true;
			HttpURLConnection httpURLCon = null;
			String oldSessionID = null;
			
			this.duration = System.currentTimeMillis();
			
			try {
				
				log_debug5("execute URL = \""+url.toString() +"\"  PORT = \""+url.getPort()+"\"");
				
				httpURLCon = (HttpURLConnection) url.openConnection();
				httpURLCon.setDoOutput(true);
        httpURLCon.setRequestMethod(this.requestMethod);
        httpURLCon.setUseCaches(false);
        httpURLCon.setDoInput(true);
				
		//-------------------------------------------------------
        // Step-Counter um eins erhhen
      	// wenn dies der erste Step des Recordings ist,
      	// dann auf Method POST mit SessionID im Cookie prfen und in diesem Fall warnen
        if ( caller.get_next_stepc()==1 ) {
        	if ( this.getRequestMethod().equals("POST") &&
							 null != (oldSessionID = this.identifySessionID((Map)header, caller.getSessionIDKey())) )
        		this.log_warn("(login?) First step request-method is POST and SessionID is already set - request-method GET expected");
        }
        
        // die aufgenommene SessionID (aus dem Recording) ermitteln
        if ( caller.getSessionIDKey() != null ) {
        	
        	// SessionID aus dem Cookie im Header extrahieren
        	oldSessionID = this.identifySessionID( (Map)header, caller.getSessionIDKey() );
					if (oldSessionID == null)
						this.log_debug4("SessionIDKey: \""+ caller.getSessionIDKey() +"\" not found in header");
        }
		//-------------------------------------------------------
				
				int responseCode = httpURLCon.getResponseCode();
				
				log_debug5("HTTP Response - StepNr: "+this.getNr() + " --> Code " + responseCode + ", Message: \"" +  httpURLCon.getResponseMessage() + "\"");
				
	      if (responseCode < 200  ||  responseCode >= 300)
					throw new Exception ("error in HTTP Response --> Code " + responseCode + ", Message: \"" +  httpURLCon.getResponseMessage() + "\"");
				
	      this.responseTime = System.currentTimeMillis();
	      
		// nach dem Abfragen des Response Codes muss fr den Request eine neue Connection aufgemacht werden
				httpURLCon = (HttpURLConnection) url.openConnection();
				httpURLCon.setDoOutput(true);
        httpURLCon.setRequestMethod(this.requestMethod);
        httpURLCon.setUseCaches(false);
        httpURLCon.setDoInput(true);
				
				// Den Header setzen
				httpURLCon = this.setHeader(httpURLCon, oldSessionID);
				
				// Den Request schreiben
				this.writeRequest(httpURLCon);
				
        // SessionID wird beim ersten Request ermittelt
				if ( caller.getSessionIDKey() != null && caller.getSessionID() == null ) {
					caller.setSessionID(this.identifySessionID(httpURLCon.getHeaderFields(), caller.getSessionIDKey()));
					if (caller.getSessionID() != null)
						this.log_debug3("current SessionID: "+caller.getSessionID());
				}
				
				// die Response verarbeiten
				this.processResponse(httpURLCon);
				
				// die Dauer des gesamten Steps ermitteln
				this.duration = System.currentTimeMillis() - this.duration;
				
		} catch (UnknownHostException e) {
				
				String err_msg = "";
				if (this.getDebugLevel()>2)
				err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": \n";
				err_msg += "Unknown Host \""+ e.getMessage() +"\" in Recording "+ caller.getNr() +", Take '"+ checkpoint.getTake() +"', Checkpoint '"+ checkpoint.getName() +"', Step "+ this.getNr();
				throw new Exception(err_msg);
				
			} catch (MalformedURLException e) {
				
				success = true;
				
			} catch (Exception e) {
				
				String err_msg = "";
				if (this.getDebugLevel()>2)
					err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": \n";
				throw new Exception(err_msg +"error in Recording "+ caller.getNr() +", Take '"+ checkpoint.getTake() +"', Checkpoint '"+ checkpoint.getName() +"', Step "+ this.getNr() +": "+ e.getMessage() );
			
			} finally {
				
				if ( httpURLCon!= null ) httpURLCon.disconnect();
			}
			
			return success;
		}
		
		
		protected void setCaller(DialogTester caller) {
			this.caller = caller;
		}
		
    /**
     * @param checkpoint
     *            The checkpoint to set.
     */
    protected void setCheckpoint(DialogCheckpoint checkpoint) {
        this.checkpoint = checkpoint;
    }

    /**
     * @return Returns the requestMethode.
     */
    protected String getRequestMethod() {
        return requestMethod;
    }

    /**
     * @param requestMethod
     *            The requestMethod to set.
     */
    protected void setRequestMethod(String requestMethod) {
        if (requestMethod.equals("")) {
            this.requestMethod = "POST";
        } else {
            this.requestMethod = requestMethod;
        }
    }

		/**
		 * @return Returns the duration.
		 */
		protected long getDuration() {
			return this.duration;
		}
		
		/**
		 * @return Returns the responseTime.
		 */
		protected long getResponseTime() {
			return this.responseTime;
		}
		
    /*
     * @param request
     *            The request to set.
     *
    protected void setRequest(HashMap request) {
        this.request = request;
    }
    */
}
