/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
/*
 * Created on 25.01.2005
 * 
 * To change the template for this generated file go to Window - Preferences -
 * Java - Code Generation - Code and Comments
 */
package sos.stresstest.dialogtest;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import sos.util.SOSClassUtil;
import sos.util.SOSLogger;


/**
 * @author ur
 * Ein Take aus einem Recording. Liest alle Checkpoints und fhrt sie aus.
 *  
 */
public class DialogTake extends DialogTesterBasic {

    private String name = "";
    private boolean success;
    //private ArrayList listOfCheckpoints; // Enthlt die Checkpoints des Takes
    private Node node;
    private NodeList nodesCheckpoints = null;
    private DialogTester caller = null;
    
    /**
     * @param sosLogger_
     * @param name
     */
    protected DialogTake(SOSLogger sosLogger_, String name) {
        this.name = name;
        success = true;
        this.sosLogger = sosLogger_;
        if (this.sosLogger != null)
					this.setDebugLevel(this.sosLogger.getLogLevel());
    }

	// Fhrt einen Take aus. Ein Take besteht aus n Checkpoints.
	protected boolean execute() throws Exception {
		
		DialogCheckpoint checkpoint = null;
		DialogTesterTags tag;
		int cnt = 0;
		
		try {
			
			nodesCheckpoints = node.getChildNodes();
			
			for (int i = 0; i < nodesCheckpoints.getLength(); i++) {
				
				if (nodesCheckpoints.item(i).getAttributes() != null)
					cnt++;
			}
			
			if (cnt == 1)
				log_info("   -> Starting Take '" + name + "' with one Checkpoint");
			else
				log_info("   -> Starting Take '" + name + "' with " + cnt + " Checkpoints");
			
			
			for (int i = 0; i < nodesCheckpoints.getLength(); i++) {
				
				if (nodesCheckpoints.item(i).getAttributes() != null) {
					
					checkpoint = new DialogCheckpoint(sosLogger, nodesCheckpoints.item(i).getAttributes().getNamedItem("name").getNodeValue());
					checkpoint.setNode(nodesCheckpoints.item(i));
					checkpoint.setCaller(caller);
					checkpoint.setTake(name);
					checkpoint.setDebugLevel(this.getDebugLevel());
					
					log_debug4("Tag-Replacement Start:");
					for (int j = 0; j < caller.getTagValues().size(); j++) {
						
						tag = (DialogTesterTags) caller.getTagValues().get(j);
						log_debug4("Take des Tags=aktTake? --> "  + tag.getTake() + "=" + name);
						// Alle Replacements dieses Takes. Prfen ob aktiv
						
						if (tag.getTake().equals(name)) {
							log_debug4("StartCheckPoint=aktCheckpoint? --> "  + tag.getStartDialogCheckpoint() + "=" + checkpoint.getName());
							tag.setActive(tag.getStartDialogCheckpoint().equals(checkpoint.getName()));
							//System.out.println(i+") "+tag.getTake()+"->DialogTake.StartDialogCheckpoint: "+tag.getStartDialogCheckpoint());
						}
					}
					
					if (caller.xmlReport!=null) caller.xmlReport.resultOpenCheckpoint(checkpoint.getName());
					
					success = ( success && checkpoint.execute() );
					
					if (caller.xmlReport!=null) caller.xmlReport.resultCloseCheckpoint();
					
					log_debug4("Tag-Replacement Stop:");
					for (int j = 0; j < caller.getTagValues().size(); j++) {
						
						tag = (DialogTesterTags) caller.getTagValues().get(j);
						log_debug4("Take des Tags=aktTake? --> "  + tag.getTake() + "=" + name);
						// Alle Replacements dieses Takes. Prfen ob weiterhin aktiv
						
						if (tag.getTake().equals(name)) {
							log_debug4("StartCheckPoint<>aktCheckpoint? --> "  + tag.getStartDialogCheckpoint() + "<>" + checkpoint.getName());
							tag.setActive( !tag.getEndDialogCheckpoint().equals(checkpoint.getName()) );
							//System.out.println(i+") "+tag.getTake()+"->DialogTake.EndDialogCheckpoint: "+tag.getEndDialogCheckpoint());
						}
					}
				}
			}
			
		} catch (Exception e) {
			
			if (caller.xmlReport!=null) caller.xmlReport.setError(e.getClass().getName(), e.getMessage());
			
			String err_msg = "";
			if (this.getDebugLevel()>2)
				err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
  		throw new Exception(err_msg + e.getMessage());
		
		} finally {
			
			if (caller.xmlReport!=null)
				if ( caller.xmlReport.resultIsCheckpointOpen() ) {
					caller.xmlReport.resultCloseCheckpoint();
				}
		}
		
		return success;
	}
	
    /**
     * @param node
     *            The node to set.
     */
    protected void setNode(Node node) {
        this.node = node;
    }

    /**
     * @param caller
     *            The caller to set.
     */
    protected void setCaller(DialogTester caller) {
        this.caller = caller;
    }

    /**
     * @return Returns the name.
     */
    protected String getName() {
        return name;
    }

}
