/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package sos.stresstest.dialogtest;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.regex.Pattern;
import java.util.regex.Matcher;

import sos.util.SOSClassUtil;

/**
 * Falls der erweiterte Suchstring Variablen enthlt werden die entsprechenden Werte aus
 * dem HTML-Stringpuffer extrahiert und in globalVars - fr eine sptere Verwendung 
 * in set_Tag_value und change_tag_value - gespeichert.
 * 
 * globalVars enthlt alle Variablen der Form <$...> und die dazugehrigen Werte.
 * 
 * @author fs
 * @since 23.08.2005
 */
public class FilterMatcher {

	private String check = null;
	private String html = null;
	private int flags = 0;
	
	private String regexp = null;
	
	private HashMap vars = null;
	
	private ArrayList key = null;
	private ArrayList value = null;	
	
	private int debugLevel = 0;
	
	/**
	 * Konstruktor
	 * @param xregexp	erweiterter Regulrer Ausdruck, welcher Variablen der Form <$...> enthalten kann
	 * @param buffer	HTML-Stringbuffer
	 * @param globalVars	Referenz auf globale VariablenMap
	 */
	protected FilterMatcher(String xregexp, String buffer, HashMap globalVars) {
		
		this.check = xregexp;
		this.html = buffer;
		this.vars = globalVars;
	}
	
	/**
	 * Konstruktor
	 * @param xregexp	erweiterter Regulrer Ausdruck, welcher Variablen der Form <$...> enthalten kann
	 * @param buffer	HTML-Stringbuffer
	 * @param globalVars	Referenz auf globale VariablenMap
	 * @param flags Match flags, a bit mask that may include CASE_INSENSITIVE, MULTILINE, DOTALL, UNICODE_CASE, and CANON_EQ from Pattern 
	 */
	protected FilterMatcher(String xregexp, String buffer, HashMap globalVars, int flags) {
		
		this.check = xregexp;
		this.html = buffer;
		this.vars = globalVars;
		this.flags = flags;
	}
	
	/**
	 * substituiert in dem regexp die Variablen durch groups
	 * und erzeugt daraus einen neuen Regulren Ausdruck
	 * @return regexp
	 */
	private String eval_xregexp() throws Exception {
		
		String result = null;
		
		try {
			
			this.key = new ArrayList();
			this.value = new ArrayList();
			
			result = this.check;
			
			String var = "<\\$[\\w]+>";
			String rplmnt = "(.*?)";
			
			Pattern pattern = Pattern.compile(var);
			Matcher matcher = pattern.matcher(result);
			
			while ( matcher.find() ) {
				
				String match = matcher.group();
				key.add(match);
				result = result.replaceFirst( "<\\$"+match.substring(2), rplmnt);
			}
			
		} catch (Exception e) {
			
			String err_msg = "";
			if (this.debugLevel>2)
				err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
  		throw new Exception(err_msg + e.getMessage());
		}
		
		return result;
	}
	
	private boolean search() throws Exception {
		
		Pattern pattern;
		boolean found = false;
		
		try {
			
			if (flags == 0)
				pattern = Pattern.compile(this.regexp);
			else
				pattern = Pattern.compile(this.regexp, this.flags);
			
			Matcher matcher = pattern.matcher(this.html);
			
			if ( matcher.find() ) {
				
				for (int i=1; i<=matcher.groupCount(); i++) {
					this.value.add(matcher.group(i));
				}
				
				for (int i=0; i<this.key.size(); i++) {
					this.vars.put( (String)this.key.get(i), (String)this.value.get(i) );
				}
				
				return true;
				
			}
			
		} catch (Exception e) {
			String err_msg = "";
			if (this.debugLevel>2)
				err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
  		throw new Exception(err_msg + e.getMessage());
		}
		
		return found;
	}
	
	/**
	 * Wenn der Suchstring gefunden wird, werden die Variablen in globalVars gesetzt und true zurckgeliefert,<br>
	 * ansonsten werden keine Variablen gesetzt und false zurckgeliefert.
	 * @return found
	 */
	protected boolean find() throws Exception {
		
		boolean search = false;
		
		try {
			
			if ( this.regexp == null )
				this.regexp = this.eval_xregexp();
			
			search = this.search(); 
			
		} catch (Exception e) {
			String err_msg = "";
			if (this.debugLevel>2)
				err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
  		throw new Exception(err_msg + e.getMessage());
		}
		
		return search;
	}
	
	protected String getSearchString() throws Exception {
		
		String searchString = null;
		
		try {
			
			if ( this.regexp == null )
				this.regexp = this.eval_xregexp();
			
			searchString = this.regexp;
			
		} catch (Exception e) {
			String err_msg = "";
			if (this.debugLevel>2)
				err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
  		throw new Exception(err_msg + e.getMessage());
		}
		
		return searchString;
	}
	
	/**
	 * zeigt die gesetzten Variablen
	 */
	protected String showVars() throws Exception {
		
		String result = "";
		
		try {
			
			for (int i=0; i<this.key.size(); i++) {
				result += " (" + (String)this.key.get(i) + " => " + (String)this.value.get(i) + ")";
			}
			
			result = "["+result+" ]";
			
		} catch (Exception e) {
			String err_msg = "";
			if (this.debugLevel>2)
				err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
  		throw new Exception(err_msg + e.getMessage());
		}
		
		return result;
	}

	protected void setDebugLevel(int debugLevel) {
		this.debugLevel = debugLevel;
	}
	
	/*
	public static void main(String[] args) {
		
		String _xregexp = "<a class=\"<$class>\" href=\"useradmin\\?user_id=<$id>\">";
		String _buffer = "XXXX<a class=\"tree\" href=\"USERADMIN?user_id=473\">YYYYYY"; 
		HashMap _globals = new HashMap();
		
		try {
			FilterMatcher fm = new FilterMatcher(_xregexp, _buffer, _globals, Pattern.CASE_INSENSITIVE);
			
			System.out.println("suche "+_xregexp);
			System.out.println("   => "+fm.getSearchString());
			System.out.println("in    "+_buffer);
			
			if ( fm.find() )
				System.out.println("gefunden");
			else
				System.out.println("nicht gefunden");
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		System.out.println( "globals -> "+_globals.toString() );
	}
	*/
}
