/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package sos.stresstest.dialogtest;

import sos.stresstest.dialogtest.DialogTester;
import sos.stresstest.dialogtest.DialogTesterBasic;
import sos.settings.SOSProfileSettings;
import sos.settings.SOSSettings;
import sos.connection.SOSConnection;

import sos.util.SOSClassUtil;
import sos.util.SOSDate;
import sos.util.SOSLogger;
import sos.util.SOSStandardLogger;

import java.util.regex.Pattern;
import java.util.regex.Matcher;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Properties;
import java.util.StringTokenizer;

import java.io.File;
import java.lang.reflect.Method;


/**
 * Diese Klasse wertet eine ini-Datei aus und<br> 
 * versorgt und startet pro Recording einen DialogTester.<br>
 * Die ini-Datei kann per Methode initialize<br>
 * oder per Kommandozeilenparameter "-config=" uebergeben werden.
 * 
 * @author Florian Schreiber
 * @since 16.03.2005 
 */

/*
 * - setValue/changeValue geht nicht mit stopCheckpoint => stop_checkpoint-Variante entfernt
 */

public class SOSDialogTest extends DialogTesterBasic {
	
	//private String inifile = null;
	private boolean initialized = false;
	
	// das Einstellungs-Objekt
	private SOSSettings settings = null;
	
	// die Sektion "settings" des Einstellungs-Objekts
	private HashMap sectionSettings = null;
	
	// fuer ein uebergeordentes Logger-Objekt
	private SOSLogger topLevelLogger = null;
	
	// Index des Stresstest-Threads
	// konstant innerhalb eines Threads
	private int thread_index = 0;
	
	// Counter innerhalb der gesamten Test-Session
	private static long session_counter = -1;
	
	// Counter innerhalb eines Threads
	private long thread_counter = -1;
	
	// enthlt globale Variablen der Form <$...> der ini-Datei und deren Werte
	// wird in DialogStep geschrieben
	private HashMap globalVars = null;
	
	private String logFile = "";
	
	// enthlt die Namen incl. Pfade der verwendeten Logfiles
	private HashMap usedLogfiles = null;
	
	protected XMLReport xmlReport = null;
	private String reportFilename = null;
	
	// bildet von diversen bool-aeqivalenten Strings auf "true" bzw. "false" ab
	private static Hashtable BOOL = new Hashtable();
	
	static {
		
		SOSDialogTest.BOOL.put("true", "true");
		SOSDialogTest.BOOL.put("false", "false");
		SOSDialogTest.BOOL.put("j", "true");
		SOSDialogTest.BOOL.put("ja", "true");
		SOSDialogTest.BOOL.put("y", "true");
		SOSDialogTest.BOOL.put("yes", "true");
		SOSDialogTest.BOOL.put("n", "false");
		SOSDialogTest.BOOL.put("nein", "false");
		SOSDialogTest.BOOL.put("no", "false");
		SOSDialogTest.BOOL.put("on", "true");
		SOSDialogTest.BOOL.put("off", "false");
		SOSDialogTest.BOOL.put("1", "true");
		SOSDialogTest.BOOL.put("0", "false");
	}
	
	
	/**
	 *	Standard-Konstruktor
	 */
	public SOSDialogTest() {
	}
	
	/**
	 * Wenn ein topLevel-Programm einen eigenen Logger bereitstellt,<br>
	 * kann dieser ueber diesen Konstruktor uebergeben werden.<br>
	 * Ansonsten wuerde DialogTester einen eigenen Logger erzeugen.
	 * Wenn dieser Konstruktor verwendet wird,<br>
	 * werden die Werte der ini-Eintraege "dialogtest_log" und "debug_level" ignoriert.
	 * @param sosLogger
	 */
	public SOSDialogTest(SOSLogger sosLogger) {
		
		this.topLevelLogger = sosLogger;
	}
	
	/**
	 * Konstruktor fr Verwendung im Stresstest
	 * @param sosLogger	der Logger vom Stresstest
	 * @param thread_index	StresstestThread.get_thread_index(), der index des jeweiligen Threads
	 */
	public SOSDialogTest(SOSLogger sosLogger, int thread_index) {
		
		this.topLevelLogger = sosLogger;
		this.thread_index = thread_index;
	}
	
	/**
	 * Setzt die Einstellungen fr den Dialogtest
	 * 
	 * @param sosSettings das Einstellungsobjekt
	 * @throws Exception
	 */
	public void setSettings(SOSSettings sosSettings) {
		
		this.settings = sosSettings;
	}
	
	/**
	 * initialisiert die Testumgebung.<br>
	 * Muss vor execute aufgerufen werden.
	 * 
	 * @param sosSettings das Einstellungsobjekt
	 * @throws Exception
	 */
	public void initialize(SOSSettings sosSettings) throws Exception {
		
		this.setSettings(sosSettings);
		this.initialize();
	}
	
	/**
	 * initialisiert die Testumgebung.<br>
	 * Muss vor execute aufgerufen werden.
	 * 
	 * @throws Exception
	 */
	public void initialize() throws Exception {
		
		try {
			
			if ( this.settings==null ) {
				throw new Exception("no configuration available");
			}
			
			this.globalVars = new HashMap();
			this.usedLogfiles = new HashMap();
			this.sectionSettings = new HashMap();
			
			/* info enthlt die Eintrge
			 * "section_settings" - Werte: "exists" wenn die Sektion exisitiert,
			 * oder null wenn die Sektion gar nicht existiert
			 * 
			 * und alle Eintrge der Sektion [settings]
			 * als Wert jeweils null wenn der Eintrag einen Wert hatte, oder "default",
			 * wenn der default-Wert verwendet wird
			 */
			HashMap info = new HashMap(); 
			
			// Sektion [settings] einlesen
			String warning = "";
			warning = this.readSectionSettings(this.settings, this.sectionSettings, info);
			
			/*
			 * - Logger setzen
			 * - alte log-Dateien mssen evtl. berschrieben werden
			 * - Ausgabe der Eintrge, die gesetzt sind
			 */
			
// Logger initialisieren
			
			if (this.topLevelLogger != null) {
				
				this.sosLogger = this.topLevelLogger;
				this.setDebugLevel(this.topLevelLogger.getLogLevel());
				
			} else if (this.topLevelLogger == null) {
				
				this.logFile = this.substitute((String)this.sectionSettings.get("dialogtest_log"), "SOSDialogTest");
				
				// wenn ein Logfile benutzt werden soll
				if ( !this.logFile.equals("") ) {
					
					// Namen des ersten verwendeten Logfiles speichern
					this.putUsedLogfile(this.logFile);
					
					String separator = "";
					if ( this.logFile.matches(".*[\\\\].*") ) separator = "\\";
					if ( this.logFile.matches(".*[/].*") ) separator = "/";
					
					String dir = this.logFile.substring(0, logFile.lastIndexOf(separator));
					File logDir = new File(dir); 
					if ( !logDir.canRead() )
						throw new Exception("log directory not found or not readable: \"" + logDir.getAbsolutePath() + "\"");
					
			  	// logfile vorher lschen, wenn append_log auf false steht
					if ( SOSDialogTest.getBoolean((String)this.sectionSettings.get("append_log")).equals("false") ) {
						File f = new File( this.logFile );
						try {
			  			if ( f.exists() ) f.delete();
						} catch (Exception e) {
							throw new Exception("cannot delete old logfile \"" + f.getAbsolutePath() + "\": "+e.getMessage());
						}
					}
					
					Integer dl = new Integer( (String)this.sectionSettings.get("debug_level") );
					this.setDebugLevel( dl.intValue() );
					
					// logging ins Logfile
					this.sosLogger = (SOSLogger)new SOSStandardLogger(this.logFile, this.getDebugLevel());
				}
			} else {
				// es findet kein Logging statt, sosLogger bleibt null
			}
			
			
			this.log_info("SOSDialogTest started");
			this.log_info("");
			
			/*	entry:						default value:
					enable_checks			true
					connection_ini		""
					debug_level				0
					dialogtest_log		""
					append_log				false
					sessionid_key			""
					http_proxyhost		""
					http_proxyport		""
					report						""
			*/
			
			if ( ((String)info.get("section_settings")).equals("exists") )
				this.log_info("read configuration: section [settings]");
			else
				this.log_debug3("read configuration: section [settings] not available. Default values are used");
			
			String entry;
			
			entry = "report";
			if ( info.get(entry)==null ) {
				String file = (String)this.sectionSettings.get(entry);
				
				if ( !file.equals("") ) {
					File f = new File(file);
					try {
						f.createNewFile();
					} catch (Exception e) {
						throw new Exception("cannot create report file '"+f.getAbsolutePath()+"': "+e.getMessage());
					}
					
					this.reportFilename = file;
					// Report beginnen
					this.xmlReport = new XMLReport(file);				
				}
			}
			
			entry = "enable_checks";
			if ( info.get(entry)==null ) {
				boolean isTrue = ( (String)SOSDialogTest.getBoolean((String)this.sectionSettings.get(entry)) ).equals("true");
				this.log_info("[settings]->" + entry + ": checks " + ((isTrue) ? "enabled" : "disabled") );
				if (this.xmlReport!=null) this.xmlReport.configAddSettingsParameter(entry, ((isTrue) ? "true" : "false"));
			} else
				this.log_debug3("[settings]->" + entry + ": checks enabled (default)");
			
			entry = "connection_ini";
			if ( info.get(entry)==null ) {
				this.log_info("[settings]->" + entry + ": " + ((String)this.sectionSettings.get(entry)) );
				if (this.xmlReport!=null) this.xmlReport.configAddSettingsParameter(entry, ((String)this.sectionSettings.get(entry)));
			} else
				this.log_debug3("[settings]->" + entry + ": not used (default)");
			
			entry = "debug_level";
			if ( this.topLevelLogger!=null ) {
				if ( info.get(entry)==null )
					this.log_info("[settings]->" + entry + ": ignored (use external logger at level "+this.topLevelLogger.getLogLevel()+")");
			} else {
				if ( info.get(entry)==null ) {
					// Evtl. eine Warnung fuer das Debug-Level loggen
					if ( !warning.equals("") )
						this.log_warn(warning);
					this.log_info("[settings]->" + entry + ": " + ((String)this.sectionSettings.get(entry)) );
					if (this.xmlReport!=null) this.xmlReport.configAddSettingsParameter(entry, ((String)this.sectionSettings.get(entry))); 
				} else
					this.log_debug3("[settings]->" + entry + ": " + ((String)this.sectionSettings.get(entry)) + " (default)");
			}
			
			entry = "dialogtest_log";
			if ( this.topLevelLogger!=null ) {
				if ( info.get(entry)==null )
					this.log_info("[settings]->" + entry + ": ignored (use external logger at level "+this.topLevelLogger.getLogLevel()+")");
			} else {
				if ( info.get(entry)==null ) {
					this.log_info("[settings]->" + entry + ": " + ((String)this.sectionSettings.get(entry)) );
					if (this.xmlReport!=null) this.xmlReport.configAddSettingsParameter(entry, ((String)this.sectionSettings.get(entry)));
				} else
					this.log_debug3("[settings]->" + entry + ": no log file(s) (default)");
			}
			
			entry = "append_log";
			if ( this.topLevelLogger!=null ) {
				if ( info.get(entry)==null )
					this.log_info("[settings]->" + entry + ": ignored ");
			} else {
				if ( info.get(entry)==null ) {
					boolean isTrue = ( (String)SOSDialogTest.getBoolean((String)this.sectionSettings.get(entry)) ).equals("true");
					this.log_info("[settings]->" + entry + ": " + ((isTrue) ? "append log to logfiles(s)" : "overwrite logfile(s)") );
					if (this.xmlReport!=null) this.xmlReport.configAddSettingsParameter(entry, ((isTrue) ? "true" : "false"));
				} else
					this.log_debug3("[settings]->" + entry + ": overwrite logfile(s) (default)");
			}
			
			entry = "sessionid_key";
			if ( info.get(entry)==null ) {
				this.log_info("[settings]->" + entry + ": use new SessionID (key: \"" + ((String)this.sectionSettings.get(entry)) + "\")");
				if (this.xmlReport!=null) this.xmlReport.configAddSettingsParameter(entry, ((String)this.sectionSettings.get(entry)));
			} else
				this.log_debug3("[settings]->" + entry + ": use recorded SessionID (default)");
			
			entry = "http_proxyhost";
			if ( info.get(entry)==null ) {
				this.log_info("[settings]->" + entry + ": use host \"" + ((String)this.sectionSettings.get(entry)) + "\"");
				if (this.xmlReport!=null) this.xmlReport.configAddSettingsParameter(entry, ((String)this.sectionSettings.get(entry)));
			} else
				this.log_debug3("[settings]->" + entry + ": no proxy used (default)");
			
			entry = "http_proxyport";
			if ( info.get(entry)==null ) {
				this.log_info("[settings]->" + entry + ": use port \"" + ((String)this.sectionSettings.get(entry)) + "\"");
				if (this.xmlReport!=null) this.xmlReport.configAddSettingsParameter(entry, ((String)this.sectionSettings.get(entry)));
			} else
				this.log_debug3("[settings]->" + entry + ": no proxy used (default)");
			
			if ( !((String)this.sectionSettings.get("http_proxyhost")).equals("") ) {
				
				System.setProperty("http.proxyHost", (String)this.sectionSettings.get("http_proxyhost"));
				System.setProperty("http.proxyPort", (String)this.sectionSettings.get("http_proxyport"));
				System.setProperty("http.proxySet", "true");
			}
			
			// information ber xml report
			entry = "report";
			String file = (String)this.sectionSettings.get(entry);				
			if ( !file.equals("") )
				this.log_info("[settings]->" + entry + ": "+file);
			else
				this.log_debug3("[settings]->" + entry + ": create no report (default)");
			
			// Die Initialisierung war erfolgreich
			this.initialized = true;
			
		} catch (Exception e) {
			
			String err_msg = "error occurred during initialisation:\n" + e.getMessage();
			throw new Exception(err_msg);
		}
	}
	
	/**
	 * Liest die Section [settings] evtl. mit default-Werten aufbereitet ein.<br>
	 * Die HashMap secSettings wird angelegt und gefllt.
	 * 
	 * @param profile Das Einstellungsobjekt
	 * @param secSettings Referenz auf HashMap
	 * @param info HashMap um Informationen ber die Sektion [settings] zu speichern, damit diese spter ausgegeben werden knnen
	 * @return Warnung
	 * @throws Exception
	 */
	private String readSectionSettings(SOSSettings profile, HashMap secSettings, HashMap info) throws Exception {
		
		if ( profile==null ) throw new Exception("no configuration available");
		
		if ( secSettings==null ) secSettings = new HashMap();
		if ( info==null ) info = new HashMap();
		
		String warning = "";
		String location = "[settings]->";
		
		String entry = null;
		String value = null;
		
		Pattern pattern = null;
		Matcher matcher = null;
		String sectionName = null;
		Properties section = null;
		
		/*	entry:						default value:
				enable_checks			true
				connection_ini		""
				debug_level				0
				dialogtest_log		""
				append_log				false
				sessionid_key			""
				http_proxyhost		""
				http_proxyport		""
		*/
		
		try {
			
// Sektion [settings] im Profil suchen
			ArrayList allSectionNames = this.settings.getSections();
			
			for (int i=0; i<allSectionNames.size(); i++) {
				
				sectionName = allSectionNames.get(i).toString();
				pattern = Pattern.compile("settings", Pattern.CASE_INSENSITIVE);
				matcher = pattern.matcher(sectionName);
				
				// settings gefunden
				if ( matcher.matches() ) {
					
					section = this.settings.getSection(sectionName);
					break; // Suche beenden
				}
			}
			
// wenn die Sektion [settings] nicht existiert,
// werden in der HashMap ausschliesslich default-Werte verwendet
			if (section == null) {
				section = new Properties();
				
// die Sektion existiert
			} else {
				info.put("section_settings", "exists");
			}
			
// Eintrge aus der Sektion lesen:
			
			entry = "enable_checks";
			value = section.getProperty(entry);
			if (value==null) info.put(entry, "default");
			value = section.getProperty(entry, "true");
			if (SOSDialogTest.getBoolean(value) == null)
				throw new Exception(location+entry+": illegal value \"" + value + "\"");
			secSettings.put(entry, value);
			
			
			entry = "connection_ini";
			value = section.getProperty(entry);
			if (value==null) info.put(entry, "default");
			value = section.getProperty(entry, "");
			if ( !value.equals("") ) {
				File f = new File(value);
				if ( !f.canRead() )
					throw new Exception(location+entry+": file not found or not readable: \"" + f.getAbsolutePath() + "\"");
			}
			secSettings.put(entry, value);
			
			
			entry = "debug_level";
			value = section.getProperty(entry);
			if (value==null) info.put(entry, "default");
			value = section.getProperty(entry, "0");
			int v=-1;
			try {
				v = new Integer(value).intValue();
				if ( !(0<=v && v<=9) ) {
					warning += location+entry+": illegal value \"" + value + "\" - default value 0 is used\n";
					value = "0";
				}
			} catch (Exception e) {
				warning += location+entry+": illegal value \"" + value + "\" - default value 0 is used\n";
				value = "0";
			}
			secSettings.put(entry, value);
			
			
			entry = "dialogtest_log";
			value = section.getProperty(entry);
			if (value==null) info.put(entry, "default");
			value = section.getProperty(entry, "");
			secSettings.put(entry, value);
			
			
			entry = "append_log";
			value = section.getProperty(entry);
			if (value==null) info.put(entry, "default");
			value = section.getProperty(entry, "false");
			if (SOSDialogTest.getBoolean(value) == null)
				throw new Exception(location+entry+": illegal value \"" + value + "\"");
			secSettings.put(entry, value);
			
			
			entry = "sessionid_key";
			value = section.getProperty(entry);
			if (value==null) info.put(entry, "default");
			value = section.getProperty(entry, "");
			secSettings.put(entry, value);
			
			
			entry = "http_proxyhost";
			value = section.getProperty(entry);
			if (value==null) info.put(entry, "default");
			value = section.getProperty(entry, "");
			boolean http_proxyHost = !value.equals("");
			secSettings.put(entry, value);
			
			
			entry = "http_proxyport";
			value = section.getProperty(entry);
			if (value==null) info.put(entry, "default");
			value = section.getProperty(entry, "");
			boolean http_proxyPort = !value.equals("");
			secSettings.put(entry, value);
			
			// wenn ein Eintrag gesetzt ist, muss auch der andere gesetzt sein
			if ( http_proxyHost || http_proxyPort ) {
				
				if ( !http_proxyHost ) throw new Exception(location+": \"http_proxyHost\" missing (\"http_proxyPort\" is set)");
				if ( !http_proxyPort ) throw new Exception(location+": \"http_proxyPort\" missing (\"http_proxyHost\" is set)");
			}
			
			
			entry = "report";
			value = section.getProperty(entry);
			if (value==null) info.put(entry, "default");
			value = section.getProperty(entry, "");
			secSettings.put(entry, value);
			
		} catch (Exception e) {
			String err_msg = "";
			if (this.getDebugLevel() > 2)
				err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
			throw new Exception(err_msg + e.getMessage());
		}
		
		return warning;
	}
	
	/**
	 * Startet den Dialog-Test.<br>
	 * Vorher muss initialize() aufgerufen worden sein.
	 */
	public void execute() throws Exception {
		
		try {
			
			if (this.initialized == false)
				throw new Exception("no initialization was done via initialize()");
			
			this.log_info("");
			this.log_info("start execution of recording(s)");
			
			// Anzeige der aktuellen Proxy-Einstellung
			String proxy = System.getProperty("http.proxyHost");
			if (proxy==null || proxy.equals(""))
				log_debug3("http.proxyHost: undefined");
			else
				log_debug3("http.proxyHost: "+proxy);
			
			proxy = System.getProperty("http.proxyPort");
			if (proxy==null || proxy.equals(""))
				log_debug3("http.proxyPort: undefined");
			else
				log_debug3("http.proxyPort: "+proxy);
			
			this.processInifile();
			
		} catch (Exception e) {
			try {this.log_info("");} catch (Exception exc) {};
			this.catch_log("SOSDialogTest terminated with errors\n");
			
			String err_msg = "";
			if (this.getDebugLevel() > 2)
				err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
			
			throw new Exception("error occurred executing recording(s): " + err_msg + e.getMessage());
		}
	}
	
//****************************************************************************************
	
	/**
	 * Ersetzt alle Vorkommen von Variablen eines ini-Eintrags durch ihren aktuellen Wert.<br>
	 * Ersetzt werden die Variablen:<br>
	 * 
	 * @param template
	 * @return Resultat mit ersetzten Variablen
	 */
	private String substitute(String template, String id) throws Exception {
		
		if (template==null || id==null) return null;
		
		String result = template;
		Pattern pattern;
		Matcher matcher;
		String var; // regular expression
		Pattern sub;
		String regexp; // regular expression
		String rplmnt; // replacement
		
		try {
			if ( !id.equals("SOSDialogTest") )	// hier kann noch kein Logger existieren
				this.log_debug5("SOSDialogTest.substitute("+template+" , "+id+")");
			
			// erste Prfung, ob es was zu ersetzen gibt
			if (template.lastIndexOf('<') == -1) return template;
			
			regexp = "<#>";
			rplmnt = id;	// hier kommt die Nummer des Recordings rein
			pattern = Pattern.compile(regexp);
			matcher = pattern.matcher(result);
			if ( matcher.find() ) {
				result = matcher.replaceAll(rplmnt);
				if ( !id.equals("SOSDialogTest") )	// hier kann noch kein Logger existieren
					this.log_debug5("replace all \""+regexp+"\" with \""+rplmnt+"\"");
			}
			
			// Datum ersetzen
			result = this.substitute_date(result);
			
			regexp = "<time>";
			rplmnt = String.valueOf( System.currentTimeMillis() );
			pattern = Pattern.compile(regexp);
			matcher = pattern.matcher(result);
			if ( matcher.find() ) {
				result = matcher.replaceAll(rplmnt);
				this.log_debug5("replace all \""+regexp+"\" with \""+rplmnt+"\"");
			}
			
			regexp = "<thread_index>";
			rplmnt = String.valueOf( this.get_thread_index() );
			pattern = Pattern.compile(regexp);
			matcher = pattern.matcher(result);
			if ( matcher.find() ) {
				result = matcher.replaceAll(rplmnt);
				this.log_debug5("replace all \""+regexp+"\" with \""+rplmnt+"\"");
			}
			
			// substitution von thread_countern: <thread_current_value> und <thread_next_value>
			result = this.substitute_threadCounter(result);
			
			regexp = ".*<session_next_value>.*";
			pattern = Pattern.compile(regexp);
			matcher = pattern.matcher(result);
			
			if ( matcher.matches() ) {
				
				var = "<session_next_value>";
				sub = Pattern.compile(var);
				
				while ( matcher.matches() ) {
					
					rplmnt = String.valueOf( this.get_session_next_value() );
					result = sub.matcher(result).replaceFirst(rplmnt);
					this.log_debug5("replace first \""+var+"\" with \""+rplmnt+"\"");
					matcher = pattern.matcher(result);
				}
			}
			
			if ( !id.equals("SOSDialogTest") )	// hier kann noch kein Logger existieren
				this.log_debug4("SOSDialogTest.substitute("+template+" , "+id+") result="+result);
		
		} catch (Exception e) {
			String err_msg = "";
			if (this.getDebugLevel()>2)
				err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
  		throw new Exception(err_msg + e.getMessage());
		}
		
		return result;
	}
	
	private String substitute_date(String template) throws Exception {
		
		if (template==null) return null;
		
		Pattern p;
		Matcher m;		
		String rplmnt;
		String result = null;
		
		try {
		
			// erste Prfung, ob es was zu ersetzen gibt
			if (template.lastIndexOf("<date") == -1) return template;
			
			result = template;
			
			String regexp = "<date(,[\\s]*([^>]+))?>";
			Pattern pattern = Pattern.compile(regexp);
			Matcher matcher = pattern.matcher(result);
			
			while ( matcher.find() ) {
				
				// <date>
				if (matcher.group(1) == null) {
					rplmnt = SOSDate.getCurrentDateAsString(); // Datum im ISO-Format
					
				// <date,modifier>
				} else {
					rplmnt = SOSDate.getCurrentDateAsString(matcher.group(2)); // Datum im selbst definierten Format
				}
				
				p = Pattern.compile(regexp);
				m = p.matcher(result);	
				result = m.replaceFirst(rplmnt);
				this.log_debug5("replace first \""+matcher.group(0)+"\" with \""+rplmnt+"\"");
			}
		
		} catch (Exception e) {
			
			String err_msg = "";
			if (this.getDebugLevel()>2)
				err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
  		throw new Exception(err_msg + e.getMessage());
		}
	
		return result;
	}
	
	private String substitute_threadCounter(String template) throws Exception {
		
		if (template==null) return null;
		
		Pattern p;
		Matcher m;
		String rplmnt;
		String result = null;

		try {
		
			// erste Prfung, ob es was zu ersetzen gibt
			if (template.lastIndexOf("<thread_") == -1) return template;
			
			result = template;
			
			String regexp = "<thread_next_value>|<thread_current_value>";
			
			Pattern pattern = Pattern.compile(regexp);
			Matcher matcher = pattern.matcher(result);
			while ( matcher.find() ) {
				
				if ( matcher.group().equals("<thread_next_value>") ) {
					p = Pattern.compile("<thread_next_value>");
					rplmnt = String.valueOf( this.get_thread_next_value() );
				} else {
					p = Pattern.compile("<thread_current_value>");
					rplmnt = String.valueOf( this.get_thread_current_value() );
				}
				m = p.matcher(result);	
				result = m.replaceFirst(rplmnt);
				this.log_debug5("replace first \""+matcher.group()+"\" with \""+rplmnt+"\"");
			}
			
		} catch (Exception e) {
			String err_msg = "";
			if (this.getDebugLevel()>2)
				err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
  		throw new Exception(err_msg + e.getMessage());
		}
		
		return result;
	}
	
	private void processInifile() throws Exception {
		
		int status;
		String sectionName = null;
		Properties section = null;
		String value = null;
		String err_msg = "";
		
		try {
			
			ArrayList resultSet = new ArrayList();
			
			// alle Recordings durchgehen
			sectionName = "recording_";
			int i=1;
			
			while ( !(section = this.settings.getSection(sectionName+i)).isEmpty() ) {
				
				this.log_info("-----------------------------------------------------------------------------------------------------------");
				
				// ist [recording_i] enabled?
				if (section.containsKey("enabled")) {
					
					value = (String)section.get("enabled");
					
					if ( SOSDialogTest.getBoolean(value) == null )
						throw new Exception("["+sectionName+i+"]->enabled: illegal value \""+value+"\"");
					
					if (SOSDialogTest.getBoolean(value).equals("false")) {
						
						this.log_info("["+sectionName+i+"]->enabled=false => \""+(String)section.get("recording_xml")+"\" skipped");
						resultSet.add("skipped");
						i++;
						continue; // Recording ist deaktiviert => Recording ueberspringen
					}
				}
				
				// Recording-Datei ueberpruefen
				if (! section.containsKey("recording_xml")) {
					
					String warning = "["+sectionName+i+"]: entry \"recording_xml\" missed. \"Recording "+i+"\" skipped";
					this.log_warn(warning);
					resultSet.add("skipped");
					i++;
					continue;
				}
				
				// existiert die xml-Datei?
				value = (String)section.get("recording_xml");
				if (! new File(value).exists()) {
					throw new Exception("["+sectionName+i+"]->recording_xml: file \""+value+"\" does not exist");
				}
				
				// alles ok! Recording wird ausgefuehrt
				status = this.executeRecording(i);
	
				if (status == 0)
					resultSet.add("success");
				else if (status == 1)
					resultSet.add("listing");
				else
					resultSet.add("error !");
	
				i++;
			}
			
			this.log_info("");
			this.log_info("SOSDialogTest terminated successfully");
			this.log_info("");
			this.log_info("----------------------------------------------------------");
			this.log_info("summary:");
			this.log_info("");
			for (int n=0; n<resultSet.size(); n++) {
				this.log_info("Recording "+(n+1)+": "+(String)resultSet.get(n));
			}
			this.log_info("==========================================================");
		
		} catch (Exception e) {
			
			if (this.getDebugLevel()>2)
				err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
			
			err_msg += e.getMessage();
			
		} finally {
			
			try {
				if (this.xmlReport!=null) this.xmlReport.createReport();
			} catch (Exception ex) {
				err_msg += "\n"+ex.getMessage();
			}
			
			if ( !err_msg.equals("") )
				throw new Exception(err_msg);
		}
		
	}
	
	private int executeRecording(int nr) throws Exception {
		
		int status = -1; // als Fehler voreingestellt
		
		SOSConnection recordingConnection = null;
		
		boolean list = false;
		
		try {
			
			DialogTester dt = new DialogTester();
			this.log_debug4("DialogTester object created");
			
			dt.setNr(nr);
			
			Properties section = this.settings.getSection("recording_"+nr);
			String location = "[recording_"+nr+"]->";
			String value;
			String entry;
			
			this.log_info("read data of Recording "+nr);
			
			list = this.isSetTrue(section, location, "list");
			
			if ( !list && this.xmlReport!=null ) this.xmlReport.configOpenRecording(nr);
			
			// recording_xml
			entry = "recording_xml";
			String recording = (String)section.get(entry);
			this.log_info(location+entry+" = "+recording);
			if ( !list && this.xmlReport!=null ) this.xmlReport.configAddRecordingParameter(entry, recording);
			
			// enable_checks setzen
			entry = "enable_checks";
			if ( ((String)this.sectionSettings.get(entry)).equals("false") ) {
				dt.setEnableChecks(false);
				this.log_info(location+entry+" = false");
			}
			
			// Logger oder Logfile setzen
			if (this.topLevelLogger != null) {
				
				dt.setSosLogger(this.topLevelLogger);
				dt.setDebugLevel(this.getDebugLevel());
				
			} else {
				
				// Debug-Level setzen
				entry = "debug_level";
				dt.setDebugLevel( new Integer((String)this.sectionSettings.get(entry)).intValue() );
				
				// Namen des Logfiles setzen
				entry = "dialogtest_log";
				String logfile = this.substitute((String)this.sectionSettings.get(entry), String.valueOf(nr));
				dt.setLogFile(logfile);
				
				this.log_info("log file of recording_"+nr+": "+logfile);
				if ( !list && this.xmlReport!=null ) this.xmlReport.configAddRecordingParameter(entry, logfile);
			}
			
			// SessionID-Key
			entry = "sessionid_key";
			value = (String)this.sectionSettings.get(entry);
			if (value != null && !value.equals("")) {
				dt.setSessionIDKey(value);
			}
			
			// Connection
			entry = "connection_ini";
			this.log_debug5("initialize Recording "+nr+" DialogTester object");
			
			// wenn kein bergeordneter Logger existert drfen alte Logfiles berschrieben werden
			// nachschauen, ob das Logfile in Spe nicht verwendet wird und append_log auf false steht
			// wenn nicht in Verwendung => altes Logfile lschen, wenn vorhanden 
			if ( this.topLevelLogger == null ) {
				if ( SOSDialogTest.getBoolean((String)this.sectionSettings.get("append_log")).equals("false") ) {
					if ( this.usedLogfiles.get(dt.getLogFile())==null ) {
						File f = new File(dt.getLogFile());
						if ( f.exists() ) f.delete();
						this.log_debug3("delete old logfile \""+dt.getLogFile()+"\"");
					}
				}
			}
			
			// Namen des Logfiles in die Liste der verwendeten Logfiles eintragen
			if ( !dt.getLogFile().equals("") )
				this.putUsedLogfile(dt.getLogFile());
			
			dt.init((String)this.sectionSettings.get(entry));
			
			// die Connection halten, falls sie per finally geschlossen werden muss
			recordingConnection = dt.getConn();
			
			// takes setzen
			entry = "takes";
			String val;
			String message = "";
			value = (String)section.get(entry);
			if ( value != null && !value.equals("") ) {
				
				StringTokenizer st = new StringTokenizer(value, ",");
				while (st.hasMoreTokens()) {
					val = st.nextToken().trim();
					dt.addTake( val );
					message += val+" ";
				}
				this.log_info(location+entry+" = "+message);
				if ( !list && this.xmlReport!=null ) this.xmlReport.configAddRecordingParameter(entry, value);
			} else {
				this.log_info(location+entry+" = null  => execute all takes");
			}
			
			// URL - Search und Replace
			entry = "url_search";
			value = (String)section.get(entry);
			if ( value != null && !value.equals("") ) {
				dt.setURL_search(value);
				this.log_info(location+entry+" = "+value);
				if ( !list && this.xmlReport!=null ) this.xmlReport.configAddRecordingParameter(entry, value);
			}
			
			entry = "url_replace";
			value = (String)section.get(entry);
			if ( value != null && !value.equals("") ) {
				dt.setURL_replace(value);
				this.log_info(location+entry+" = "+value);
				if ( !list && this.xmlReport!=null ) this.xmlReport.configAddRecordingParameter(entry, value);
			}
			
			// Checker-Objekt erzeugen (Klasse erbt von SOSChecker)
			Class checkerClass = null;
			Object checkerObject = null;
			Method method = null;
			entry = "checker_class";
			value = (String)section.get(entry);
			if ( value != null && !value.equals("") ) {
				
				this.log_info(location+entry+" = "+value);
				if ( !list && this.xmlReport!=null ) this.xmlReport.configAddRecordingParameter(entry, value);
				
				checkerClass = Class.forName(value.trim());
				checkerObject = checkerClass.newInstance();
				
				this.log_debug5(value+" object created");
				
				// Connection an die Checker-Klasse uebergeben
				if (dt.getConn() != null) {
					
					method = checkerObject.getClass().getMethod( "setConnection", new Class[] { SOSConnection.class } );				
					method.invoke  ( checkerObject, new Object[] { dt.getConn() } );
					
					this.log_debug4(value+" uses connection of Recording "+nr+" DialogTester object");
				}
				
				dt.setCheckObject(checkerObject);
			}
			
			// html_to_files
			entry = "html_to_files";
			value = (String)section.get(entry);
			if ( value != null && !value.equals("") ) {
				
				char c = value.charAt( value.length()-1 );
				if ( !( c == '\\' || c=='/' ) )
					value += "/";
				
				File f = new File(value);
				if ( !f.isDirectory() ) {
					throw new Exception(location+entry+": the path \""+value+"\" does not exist");
				}
				
				dt.setPath(value);
				dt.setCopyHtmlToFile(true);
				this.log_info(location+entry+" = "+value);
				if ( !list && this.xmlReport!=null ) this.xmlReport.configAddRecordingParameter(entry, value);
			} else
				this.log_debug3(location+entry+" = null (default)");
			
			// print_header
			if ( this.isSetTrue(section, location, "print_header") ) {
				dt.setPrintHeader(true);
				this.log_info(location+"print_header = true");
			} else
				this.log_debug3(location+"print_header = false (default)");
			
			// print_request
			if ( this.isSetTrue(section, location, "print_request") ) {
				dt.setPrintRequest(true);
				this.log_info(location+"print_request = true");
			} else
				this.log_debug3(location+"print_request = false (default)");
			
			// show_html
			if ( this.isSetTrue(section, location, "show_html") ) {
				dt.setShowHtml(true);
				this.log_info(location+"show_html = true");
			} else
				this.log_debug3(location+"show_html = false (default)");
			
			// show_step	
			if ( this.isSetTrue(section, location, "show_step") ) {
				dt.setShowStep(true);
				this.log_info(location+"show_step = true");
			} else
				this.log_debug3(location+"show_step = false (default)");
			
			
			// Sektion [recording_nr_check_html] einlesen
			this.readSectionCheckHtml(dt, nr);
			
			// Sektion [recording_nr_check_html_absent] einlesen
			this.readSectionCheckHtmlAbsent(dt, nr);
			
			// Sektion [recording_nr_set_value] einlesen
			this.readSectionSetValue(dt, nr);
			
			// Sektion [recording_nr_change_value] einlesen
			this.readSectionChangeValue(dt, nr);
			
			dt.setGlobalVars( this.getGlobalVars() );
			
			// SOSChecker.initialize()
			if (checkerObject != null) {
				this.log_debug5("call "+checkerObject.getClass().getName()+".initialize()");
				
				method = checkerObject.getClass().getMethod( "initialize", new Class[ ] { DialogTester.class } );				
				method.invoke  ( checkerObject, new Object[] { dt } );
			}
			
			// list
			if ( this.isSetTrue(section, location, "list") ) {
				
				this.log_info(location+"list = true");
				this.log_info("");
				this.log_info("listing of Recording "+nr+" started");
				dt.list(recording);
				status = 1; // code fr ein listing
				this.log_info("listing of Recording "+nr+" finished");
				
			// execute
			} else {
				this.log_debug3(location+"list = false (default)");
				this.log_info("");
				this.log_info("execution of Recording "+nr+" started");
				
				if ( this.xmlReport!=null ) {
					// XML-Dokument wird bergeben
					dt.setXmlReport(this.xmlReport);
					this.xmlReport.resultOpenRecording(nr, recording, dt.getLogFile());
				}
				
				status = dt.executeDialog(recording);
				
				if ( this.xmlReport!=null ) this.xmlReport.resultCloseRecording();
				
				this.log_info("execution of Recording "+nr+" finished");
			}
			
			if (dt.getConn() != null) {
				dt.getConn().disconnect();
				this.log_debug4("recording "+nr+": connection of DialogTester object closed");
			}
			
		} catch (Exception e) {
			String err_msg = "";
			if (this.getDebugLevel()>2)
				err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
  		throw new Exception(err_msg + e.getMessage());
		
		} finally {
			
			if ( !list && this.xmlReport!=null ) {
				
				if ( this.xmlReport.configIsRecordingOpen() ) {
					this.xmlReport.configCloseRecording();
				}
				
				if ( this.xmlReport.resultIsRecordingOpen() ) {
					this.xmlReport.resultCloseRecording();
				}
			}
			
			if (recordingConnection != null)
				recordingConnection.disconnect();
		}
		
		return status;
	}
	
	/**
	 * Liest die zu dem Recording_nr gehoerende Sektion [recording_nr_check_html]
	 * und passt das entsprechende DialogTester-Objekt an 
	 * @param dt	das DialogTester-Objekt
	 * @param nr	der Index des Recordings
	 * @throws Exception
	 */
	private void readSectionCheckHtml(DialogTester dt, int nr) throws Exception {
		
		String take;
		String checkpoint;
		String value;
		
		try {
			
			String location = "[recording_"+nr+"_check_html]->";
			Properties section = this.settings.getSection("recording_"+nr+"_check_html");
			
			if (section.isEmpty()) {
				this.log_debug4("[recording_"+nr+"_check_html] is empty");
				return;
			}
			
			int i = 1;
			while ( (value = (String)section.get("check_html_"+i)) != null ) {
				
				take = (String)section.get("take_"+i);
				checkpoint = (String)section.get("checkpoint_"+i);
				
				if ( take==null && checkpoint==null ) {
					take = "system";
					checkpoint = "system";
				} else {
					if ( take==null ) throw new Exception(location+"take_"+i+" missed");
					if ( checkpoint==null ) throw new Exception(location+"checkpoint_"+i+" missed");
				}
				
				dt.checkHtml(take, checkpoint, value);
				this.log_info("check_html_"+i+" -> {"+take+", "+checkpoint+", "+value+"}");
				i++;
			}
			
		} catch (Exception e) {
			String err_msg = "";
			if (this.getDebugLevel()>2)
				err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
  		throw new Exception(err_msg + e.getMessage());
		}
	}
	
	/**
	 * Liest die zu dem Recording_nr gehoerende Sektion [recording_nr_check_html_absent]
	 * und passt das entsprechende DialogTester-Objekt an 
	 * @param dt	das DialogTester-Objekt
	 * @param nr	der Index des Recordings
	 * @throws Exception
	 */
	private void readSectionCheckHtmlAbsent(DialogTester dt, int nr) throws Exception {
		
		String take;
		String checkpoint;
		String value;
		
		try {
			
			String location = "[recording_"+nr+"_check_html_absent]->";
			Properties section = this.settings.getSection("recording_"+nr+"_check_html_absent");
			
			if (section.isEmpty()) {
				this.log_debug4("[recording_"+nr+"_check_html_absent] is empty");
				return;
			}
			
			int i = 1;
			while ( (value = (String)section.get("check_html_absent_"+i)) != null ) {
				
				take = (String)section.get("take_"+i);
				checkpoint = (String)section.get("checkpoint_"+i);
				
				if ( take==null && checkpoint==null ) {
					take = "system";
					checkpoint = "system";
				} else {
					if ( take==null ) throw new Exception(location+"take_"+i+" missed");
					if ( checkpoint==null ) throw new Exception(location+"checkpoint_"+i+" missed");
				}
				
				dt.checkHtmlAbsent(take, checkpoint, value);
				this.log_info("check_html_absent_"+i+" -> {"+take+", "+checkpoint+", "+value+"}");
				i++;
			}
			
		} catch (Exception e) {
			String err_msg = "";
			if (this.getDebugLevel()>2)
				err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
  		throw new Exception(err_msg + e.getMessage());
		}
	}
	
	/**
	 * Liest die zu dem Recording_nr gehoerende Sektion [recording_nr_set_value]
	 * und passt das entsprechende DialogTester-Objekt an 
	 * @param dt	das DialogTester-Objekt
	 * @param nr	der Index des Recordings
	 * @throws Exception
	 */
	private void readSectionSetValue(DialogTester dt, int nr) throws Exception {
		
		String tag;
		String take;
		String start;
		//String stop;
		String value;
		
		try {
			
			String location = "[recording_"+nr+"_set_value]->";
			Properties section = this.settings.getSection("recording_"+nr+"_set_value");
			
			if (section.isEmpty()) {
				this.log_debug4("[recording_"+nr+"_set_value] is empty");
				return;
			}
			
			int i = 1;
			while ( (value = (String)section.get("value_"+i)) != null ) {
				
				if ( (tag = (String)section.get("tag_"+i)) == null )
					throw new Exception(location+"tag_"+i+" fehlt");
				
				if ( (take = (String)section.get("take_"+i)) == null )
					throw new Exception(location+"take_"+i+" fehlt");
				
				if ( (start = (String)section.get("start_checkpoint_"+i)) == null)
					throw new Exception(location+"start_checkpoint_"+i+" fehlt");
				
				//stop = (String)section.get("stop_checkpoint_"+i); // kann auch null sein
				
				value = this.substitute(value, String.valueOf(nr));
				
				dt.setTagValue(tag, take, start, value);
				/*
				if (stop == null)
					dt.setTagValue(tag, take, start, value);
				else
					dt.setTagValue(tag, take, start, stop, value);
				
				this.log_info("set_value_"+i+" -> {"+tag+", "+take+", "+start+", "+stop+", "+value+"}");
				*/
				this.log_info("set_value_"+i+" -> {"+tag+", "+take+", "+start+", "+value+"}");
				i++;
			}
			
		} catch (Exception e) {
			String err_msg = "";
			if (this.getDebugLevel()>2)
				err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
  		throw new Exception(err_msg + e.getMessage());
		}
	}
	
	/**
	 * Liest die zu dem Recording_nr gehoerende Sektion [recording_nr_change_value]
	 * und passt das entsprechende DialogTester-Objekt an 
	 * @param dt	das DialogTester-Objekt
	 * @param nr	der Index des Recordings
	 * @throws Exception
	 */
	private void readSectionChangeValue(DialogTester dt, int nr) throws Exception {
		
		String tag;
		String take;
		String start;
		//String stop;
		String oldValue;
		String newValue;
		
		try {
			
			String location = "[recording_"+nr+"_change_value]->";
			Properties section = this.settings.getSection("recording_"+nr+"_change_value");
			
			if (section.isEmpty()) {
				this.log_debug4("[recording_"+nr+"_change_value] is empty");
				return;
			}
			
			int i = 1;
			while ( (newValue = (String)section.get("new_value_"+i)) != null ) {
				
				if ( (oldValue = (String)section.get("old_value_"+i)) == null )
					throw new Exception(location+"old_value_"+i+" fehlt");
				
				if ( (tag = (String)section.get("tag_"+i)) == null )
					throw new Exception(location+"tag_"+i+" fehlt");
				
				if ( (take = (String)section.get("take_"+i)) == null )
					throw new Exception(location+"take_"+i+" fehlt");
				
				if ( (start = (String)section.get("start_checkpoint_"+i)) == null)
					throw new Exception(location+"start_checkpoint_"+i+" fehlt");
				
				//stop = (String)section.get("stop_checkpoint_"+i); // kann auch null sein
				
				newValue = this.substitute(newValue, String.valueOf(nr));
				
				dt.changeTagValue(tag, take, start, oldValue, newValue);
				/*
				if (stop == null)
					dt.changeTagValue(tag, take, start, oldValue, newValue);
				else
					dt.changeTagValue(tag, take, start, stop, oldValue, newValue);
					
				this.log_info("change_value_"+i+" -> {"+tag+", "+take+", "+start+", "+stop+", "+oldValue+", "+newValue+"}");
				*/
				
				this.log_info("change_value_"+i+" -> {"+tag+", "+take+", "+start+", "+oldValue+", "+newValue+"}");
				i++;
			}
			
		} catch (Exception e) {
			String err_msg = "";
			if (this.getDebugLevel()>2)
				err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
  		throw new Exception(err_msg + e.getMessage());
		}
	}
	
	/**
	 * Bildet die Werte 0,1,y,n,... auf "true" oder "false" ab,<br>
	 * bzw. auf null, wenn es kein bool-aequivalenter Wert ist 
	 * @param value
	 * @return "true"/"false"/null
	 */
	public static String getBoolean(String value) {
		
		if (value == null) return null;
		
		String v = value.toLowerCase();
		return (String)SOSDialogTest.BOOL.get(v);
	}
	
	/**
	 * Liefert zurueck, ob der Wert des Eintrags dem boolschen Wert "true" entspricht
	 * @param section		die Sektion
	 * @param location	String fuer die Message der Exception
	 * @param entry			der Eintrag
	 * @return					der Wert als Boolean
	 * @throws Exception
	 */
	private boolean isSetTrue(Properties section, String location, String entry) throws Exception {
		
		if (section==null || entry==null) {
			
			String err = null;
			if (section==null) err="section ";
			if (entry==null) { if (err==null) err="entry"; else err+=", entry"; }
			
			String err_msg = "error in "+SOSClassUtil.getMethodName()+": "+err+" gleich null";
			throw new Exception("\n"+err_msg);
		}
		
		try {
		
			String value = (String)section.get(entry);
			String bval = null;
			
			if (value != null) {
				
				bval = (String)SOSDialogTest.BOOL.get(value.toLowerCase());
				
				if ( bval == null )
					throw new Exception(location+entry+": illegal value \""+value+"\"");
				else
					return bval.equals("true");
			}
			
		} catch (Exception e) {
			String err_msg = "";
			if (this.getDebugLevel()>2)
				err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
  		throw new Exception(err_msg + e.getMessage());
		}
		
		return false;
	}
	
	protected HashMap getUsedLogfiles() {
		return usedLogfiles;
	}
	
	protected void putUsedLogfile(String file) {
		this.usedLogfiles.put(file, "");
	}

	protected HashMap getGlobalVars() {
		return globalVars;
	}
	
	/**
	 * Liefert den Index des aktuellen Threads.<br>
	 * Findet nur als Stresstest-Thread Verwendung.
	 * @return Thread-Index
	 */
	private int get_thread_index() {
		return thread_index;
	}
	
	public long get_session_next_value() {
		
		return ++SOSDialogTest.session_counter;
	}
	
	public long get_thread_current_value() {
		
		return this.thread_counter;
	}
	
	public long get_thread_next_value() {
		
		return ++this.thread_counter;
	}
	
	/**
	 * @return Returns the filename of the report xml file.
	 */
	public String getReportFilename() {
	
		return this.reportFilename;
	}
	
	/**
	 * Extrahiert aus den Argumenten case-insensitive den Kommandozeilen-Parameter <i>param</i>
	 * und liefert dessen Wert zurueck.<br>
	 * Es wird der letzte Parameter extrahiert, der mit <i>param</i> uebereinstimmt.<br> 
	 * Wenn der Parameter nicht existiert wird null zurueckgeliefert.
	 * 
	 * @param param	Der Parameter nach dem gesucht wird z.B. "-config="
	 * @param args	Alle Kommandozeilen-Parameter
	 * @return Wert von param
	 */
	public static String getParamValue(String param, String[] args) throws Exception {
		
		String value = null;
		Pattern p = null;
		Matcher m = null;
		
		try {
			
			for (int i=0; i<args.length; i++) {
				
				p = Pattern.compile(param+".*", Pattern.CASE_INSENSITIVE);
				m = p.matcher(args[i]);
				
				if (m.matches())
					value = args[i].substring(param.length());
			}
			
		} catch (Exception e) {
  		
			throw new Exception("error in "+SOSClassUtil.getMethodName()+": "+e.getMessage());
		}
		
		return value;
	}
	
	public static void main(String[] args) {
		
	/*
		SOSStandardLogger out = null;
		try {
			out = new SOSStandardLogger(0);
		} catch (Exception e) {
			System.err.println("Cannot create SOSStandardLogger: "+e.getMessage());
			return;
		}
	*/
		
		SOSDialogTest test = new SOSDialogTest();
		
		try {
			
			// die bergebene ini-Datei
			String iniFile = SOSDialogTest.getParamValue("-config=", args);
			
			// hier muss geprft werden, ob es die Datei gibt
			if ( iniFile==null || iniFile.equals("") )
				throw new Exception("no configuration file was given");
			
			File f = new File(iniFile);
			if ( !f.canRead() )
				throw new Exception("configuration file not found or not readable: \"" + f.getAbsolutePath() + "\"");
			
			SOSProfileSettings profile = new SOSProfileSettings( f.getAbsolutePath() );
			
			test.initialize(profile);
			test.execute();  
			
		} catch (Exception e) {
			System.err.println(e.getMessage());
		}
	}
}
