/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// errfile.cxx                                1997 SOS GmbH Berlin
//                                             Joacim Zschimmer

#include "precomp.h"
#include "../kram/sysdep.h"

#include "../kram/sos.h"
#include "../file/absfile.h"
#include "../kram/sosopt.h"
#include "../kram/sosfield.h"
#include "../kram/soslimtx.h"


namespace sos {   

//-----------------------------------------------------------------------------------Error_file

struct Error_file : Abs_file
{
                                Error_file              ();
                               ~Error_file              ();

    void                        prepare_open            ( const char*, Open_mode, const File_spec& );
    void                        open                    ( const char*, Open_mode, const File_spec& );
  //void                        close                   ( Close_mode );

 protected:
    void                        get_record              ( Area& area );

    Fill_zero                  _zero_;
    Any_file                   _file;
};


//-----------------------------------------------------------------------------Error_file_type

struct Error_file_type : Abs_file_type
{
    virtual const char*         name                    () const { return "error"; }
    virtual Sos_ptr<Abs_file>   create_base_file        () const
    {
        Sos_ptr<Error_file> f = SOS_NEW( Error_file );
        return +f;
    }
};

const Error_file_type          _error_file_type;
const Abs_file_type&            error_file_type = _error_file_type;

//-------------------------------------------------------------------Error_file::Error_file

Error_file::Error_file()
:
    _zero_ ( this+1 )
{
}

//------------------------------------------------------------------Error_file::~Error_file

Error_file::~Error_file()
{
}

//-----------------------------------------------------------------Error_file::prepare_open

void Error_file::prepare_open( const char* parameter, Open_mode open_mode, const File_spec& spec )
{
    Sos_string filename;

    for( Sos_option_iterator opt( parameter ); !opt.end(); opt.next() )
    {
        if( opt.param() || opt.pipe() )  filename = opt.rest();
        else throw_sos_option_error( opt );
    }

    if( open_mode & out )  throw_xc( "D127" );

    _file.prepare( filename, open_mode, spec );
}

//---------------------------------------------------------------------------Error_file::open

void Error_file::open( const char*, Open_mode, const File_spec& )
{
    _file.open();
}

//---------------------------------------------------------------------Error_file::get_record

void Error_file::get_record( Area& buffer )
{
    Dynamic_area key;
    
    _file.get( &key );

    try {
        key.append( '\0' );
        char* p = strchr( key.char_ptr(), ' ' );
        if( p )  *p = '\0';  // Danach kommen Parameter, die ausgewertet werden sollten!
        throw_xc( key.char_ptr() );
    } 
    catch( const Xc& x )
    {
        x.get_text( &buffer );
        if( buffer.length() > 0  &&  buffer.char_ptr()[ buffer.length() - 1 ] == '\0' )  buffer.length( buffer.length() - 1 );
    }
}


} //namespace
