/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
#include "precomp.h"
//#define MODULE_NAME "sosmain0"
//#define COPYRIGHT   "1997 SOS GmbH Berlin"
//#define AUTHOR      "Joacim Zschimmer"

#include "../kram/sysdep.h"
#include "../kram/sysxcept.h"

#include "../kram/sosstrng.h"
#include "../kram/sos.h"
#include "../kram/sosappl.h"    // Sos_static
#include "../kram/sosprof.h"
#include "../kram/log.h"
#include "../kram/log.h"
#include "../zschimmer/log.h"

#ifdef _MSC_VER
#   include "../zschimmer/com.h"
#   include "com_simple_standards.h"
#endif

using namespace std;
namespace sos {


extern int sos_main( int, char** );    // Wird von der Applikation implementiert.


//---------------------------------------------------------------------------------DEBUG ReportHook
#if defined _DEBUG && defined SYSTEM_WIN

int DebugHook( int, char* szOutMessage, int* retval )
{
    DWORD written;

    if( zschimmer::Log_ptr log = "memory_leak_check" )     // Falls bei Programmende, nach ~Log_categories 
    {
        *log << szOutMessage << "\n";
    }
    else
        WriteFile( GetStdHandle(STD_ERROR_HANDLE), szOutMessage, strlen( szOutMessage ), &written, NULL );


    *retval = FALSE;
    return FALSE;
}

#endif
//----------------------------------------------------------------------------------------sos_main0

int sos_main0( int argc, char** argv )
{
#   if defined SYSTEM_WIN && defined _DEBUG
        
        for( int i = 1; i < argc; i++ )   // -check-memory-leak ?
        {
            const char* p = argv[ i ];

            if( strcmp( p, "-check-memory-leak" ) == 0 ) 
            {
                _CrtSetDbgFlag( _CrtSetDbgFlag(_CRTDBG_REPORT_FLAG) | _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF );   //  | _CRTDBG_CHECK_ALWAYS_DF
                _CrtSetReportHook( DebugHook );

                while( ++i < argc )  argv[ i-1 ] = argv[ i ];
                argc--;
                break;
            }
        }

#   endif

    Sos_appl appl ( false );   // init im try-Block
    int rc = 0;

    try {
        appl.init();

        #ifdef Z_DEBUG
            set_log_category_default( "memory_leak_check", true );
        #endif


        for( int i = 1; i < argc; i++ )   // -sos.ini=FILENAME ?
        {
            const char* p = argv[ i ];

            if( strncmp( p, "-sos.ini=", 9 ) == 0 ) 
            {
                //if( !strchr( p + 9, ' ' ) )                     // Nicht bei "-sos.ini=filename bla ...", Blanks im Dateinamen nicht erlaubt
                if( p[9] == '"'  &&  p[ strlen( p ) - 1 ] == '"' )
                {
                    set_sos_ini_filename( string( p + 10, strlen( p + 10 ) - 1 ) );
                }
                else
                {
                    set_sos_ini_filename( p + 9 );
                }
            }
        }

        rc = sos_main( argc, argv );

        if( rc ) {
            //SHOW_ERR( "Programm-Rckgabewert: " << rc );
        }
    }

    catch( const Xc& x )
    {
        SHOW_ERR( "Fehler " << x );
        rc = 9999;
    }

    catch( const Xc_base& )
    {
        SHOW_ERR( "Fehler " /*<< x*/ );
        rc = 9999;
    }

    catch( const zschimmer::Xc& x )
    {
        SHOW_ERR( "Fehler " << x );
        rc = 9999;
    }

    catch( const exception& x )
    {
#       if defined __BORLANDC__
            SHOW_ERR( "Exception " << __throwExceptionName
                   << "( "         << x.why().c_str()
                   << " ) in "     << __throwFileName
                   << ", Zeile "   << __throwLineNumber );
//#    elif defined SYSTEM_SOLARIS
     //     SHOW_ERR( "Unbekannte Exception: " << x.why() );
#       else
            SHOW_ERR( "Exception exception(\"" << exception_text( x ) << "\")" );
#       endif

        rc = 9999;
    }

    catch( const char* text )
    {
#       if defined __BORLANDC__
            SHOW_ERR( "Exception " << __throwExceptionName
                   << text
                   << " in "       << __throwFileName
                   << ", Zeile "   << __throwLineNumber );
#        else
            SHOW_ERR( "Exception (char*) " << text );
#       endif
    }

#ifdef _MSC_VER
    catch( const _com_error& x )
    {
        SHOW_MSG( "Fehler HRESULT=" << as_hex_string( x.Error() ) << " source=" << bstr_as_string(x.Source()) << ", " << bstr_as_string(x.Description()) );
        rc = x.Error();
    }
#endif

#if !defined _MSC_VER       // Fr MSVC++ ist auch ein Absturz eine Exception
    catch( ... )
    {
#       if defined __BORLANDC__
            SHOW_ERR( "Exception " << __throwExceptionName
                      << " in "       << __throwFileName
                      << ", Zeile "   << __throwLineNumber );
#		 elif defined SYSTEM_GNU
            SHOW_ERR( "Unbekannte Exception" );
#        elif defined SYSTEM_SOLARIS
            //SHOW_ERR( "Unbekannte Exception: " << exception_name() );
            SHOW_ERR( "Unbekannte Exception" );
#        endif
        rc = 9999;
    }
#endif

    return rc;
}

} //namespace sos
