/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: java.h 5531 2008-01-18 13:39:38Z jz $

#ifndef __ZSCHIMMER_JAVA2_H_
#define __ZSCHIMMER_JAVA2_H_


#include <vector>
#include <jni.h>
#include "threads.h"
#include "z_com.h"
#include "log.h"

namespace zschimmer {
namespace javabridge {

struct Class;
struct Method;
struct Local_jobject;

//-------------------------------------------------------------------------------------------------

string                          java_class_path_of_name     ( const string& className );

//--------------------------------------------------------------------------------------Simple_type
// Primitive Typen und Void, Object und Array

enum Simple_type 
{
    t_none    = 0,
    t_void    = 'V',
    t_boolean = 'Z',
    t_byte    = 'B',
    t_char    = 'C',
    t_short   = 'S',
    t_int     = 'I',
    t_long    = 'J',
    t_float   = 'F',
    t_double  = 'D',
    t_object  = 'L',
    t_array   = '['
};

//---------------------------------------------------------------------------------------------Type

struct Type
{
                                Type                        ( Simple_type t )                       : _simple_type( t ) { assert( t ); }
                                Type                        ( const string& class_name );
                                Type                        ( const char* class_name );

    Simple_type                 simple_type                 () const                                { return _simple_type; }
    string                      class_name                  () const                                { return _class_name; }
    string                      signature_string            () const;

    static Type                 of_type_signature_string    ( const string& signature );            // Z.B.: "I", "Ljava/lang/String;"

    static void                 check_class_name            ( const string& );
    static bool                 is_class_name               ( const string& );

  private:
    Simple_type                _simple_type;
    string                     _class_name;
};

//----------------------------------------------------------------------------------------Signature

struct Signature
{
                                Signature                   ( const string& signature_string )      : _string( signature_string ) {}

    const string&               as_string                   () const                                { return _string; }
                                operator const string&      () const                                { return _string; } 
    Type                        return_type                 () const;
    Simple_type                 return_simple_type          () const                                { return return_type().simple_type(); }
    int                         parameter_count             () const;

    static void                 check_class_signature_string( const string& signature );
    static bool                 is_class_signature_string   ( const string& signature );            // "Ljava/lang/String;" => true
    static void                 check_simple_type_char      ( char c );
    static bool                 is_simple_type_char         ( char c );                             // 'I' => true

    static Signature            of_types                    ( const Type& result_type ); 
    static Signature            of_types                    ( const Type& result_type, const Type& );
    static Signature            of_types                    ( const Type& result_type, const Type&, const Type& ); 
    static Signature            of_types                    ( const Type& result_type, const Type&, const Type&, const Type& ); 


  private:
    enum Special { special };
                                Signature                   ( Special, const char* signature_string ) : _string( signature_string ) {}

    string                     _string;
};

//---------------------------------------------------------------------------------------------Value
// Wie Jvalue, mit kleinen nderungen, um Jvalue erstmal nicht anzurhren. Jvalue sollte durch Value ersetzt werden.

struct Value : Object
{
                                Value                       ()                                      : _type( t_void ) {}
                                //Value                       ( jboolean v )                          : _type( t_void ) { assign( v ); }
                                Value                       ( bool v )                              : _type( t_void ) { assign_jboolean( v ); }
                                //Value                       ( jbyte v )                             : _type( t_void ) { assign( v ); }
                                //Value                       ( jchar v )                             : _type( t_void ) { assign( v ); }
                                Value                       ( jint v )                              : _type( t_void ) { assign_jint( v ); }
                                Value                       ( jlong v )                             : _type( t_void ) { assign_jlong( v ); }
                                Value                       ( jfloat v )                            : _type( t_void ) { assign_jfloat( v ); }
                                Value                       ( jdouble v )                           : _type( t_void ) { assign_jdouble( v ); }
                                Value                       ( jobject v )                           : _type( t_void ) { assign_jobject( v ); }
                                Value                       ( const char* v )                       : _type( t_void ) { assign_string( v ); }
                                Value                       ( const string& v )                     : _type( t_void ) { assign_string( v ); }
                                Value                       ( const Value& v )                      : _type( t_void ) { assign_value( v ); }
    virtual                    ~Value                       ();

    Value&                      operator =                  ( bool v )                              { assign_jboolean( v ); return *this; }
    //Value&                      operator =                  ( jboolean v )                          { assign( v ); return *this; }
    //Value&                      operator =                  ( jbyte v )                             { assign( v ); return *this; }
    //Value&                      operator =                  ( jchar v )                             { assign( v ); return *this; }
    Value&                      operator =                  ( jshort v )                            { assign_jshort( v ); return *this; }
    Value&                      operator =                  ( jint v )                              { assign_jint( v ); return *this; }
    Value&                      operator =                  ( jlong v )                             { assign_jlong( v ); return *this; }
    Value&                      operator =                  ( jfloat v )                            { assign_jfloat( v ); return *this; }
    Value&                      operator =                  ( jdouble v )                           { assign_jdouble( v ); return *this; }
    Value&                      operator =                  ( jobject v )                           { assign_jobject( v ); return *this; }
    Value&                      operator =                  ( const char* v )                       { assign_string( v ); return *this; }
    Value&                      operator =                  ( const string& v )                     { assign_string( v ); return *this; }
    Value&                      operator =                  ( const Value& v )                      { assign_value( v ); return *this; }

    void                        assign                      ( bool v )                              { assign_jboolean( v ); }
    //void                        assign                      ( jboolean v )                          { assign( v ); }
    //void                        assign                      ( jbyte v )                             { assign( v ); }
    //void                        assign                      ( jchar v )                             { assign( v ); }
    void                        assign                      ( jshort v )                            { assign_jshort( v ); }
    void                        assign                      ( jint v )                              { assign_jint( v ); }
    void                        assign                      ( jlong v )                             { assign_jlong( v ); }
    void                        assign                      ( jfloat v )                            { assign_jfloat( v ); }
    void                        assign                      ( jdouble v )                           { assign_jdouble( v ); }
    void                        assign                      ( jobject v )                           { assign_jobject( v ); }
    void                        assign                      ( const char* v )                       { assign_string( v ); }
    void                        assign                      ( const string& v )                     { assign_string( v ); }

    void                        assign_jboolean             ( jboolean v )                          { set_simple_type( t_boolean );  _jvalue.z = v; }
    void                        assign_jbyte                ( jbyte v )                             { set_simple_type( t_byte    );  _jvalue.b = v; }
    void                        assign_jchar                ( jchar v )                             { set_simple_type( t_char    );  _jvalue.c = v; }
    void                        assign_jshort               ( jshort v )                            { set_simple_type( t_short   );  _jvalue.s = v; }
    void                        assign_jint                 ( jint v )                              { set_simple_type( t_int     );  _jvalue.i = v; }
    void                        assign_jlong                ( jlong v )                             { set_simple_type( t_long    );  _jvalue.j = v; }
    void                        assign_jfloat               ( jfloat v )                            { set_simple_type( t_float   );  _jvalue.f = v; }
    void                        assign_jdouble              ( jdouble v )                           { set_simple_type( t_double  );  _jvalue.d = v; }
    void                        assign_jobject              ( jobject v );
    void                        assign_jstring              ( jstring v )                           { assign_jobject( v ); }
    void                        assign_string               ( const char* v )                       { assign_string( v, strlen( v ) ); }
    void                        assign_string               ( const string& v )                     { assign_string( v.data(), v.length() ); }
    void                        assign_string               ( const char* v, int length );
    void                        assign_value                ( const Value& );
    void                        steal_local_ref              ( jobject );

    void                        set_simple_type             ( Simple_type );

    const jvalue&               get_jvalue                  ()                                      { return _jvalue; }

    jobject                     as_jobject                  () const                                { check_simple_type( t_object );  return _jvalue.l; }
    bool                        as_bool                     () const                                { check_simple_type( t_boolean );  return _jvalue.b != 0; }
    string                      as_string                   () const;

    void                        check_simple_type           ( const Simple_type& ) const;
    void                        check_type                  ( const Type& ) const;
    string                      signature_string            () const;

private:
    void                    set_global_ref                  ( jobject o );

private:
    jvalue                     _jvalue;
    Simple_type                _type;
};

//-----------------------------------------------------------------------------------Parameter_list

struct Parameter_list
{
    virtual                    ~Parameter_list              ()                                      {}
    virtual int                 size                        () const                                = 0;
    virtual const jvalue*       jvalue_array                () const                                = 0;

    static const Parameter_list& empty;
};

//-------------------------------------------------------------------------------raw_parameter_list

template<int SIZE>
struct raw_parameter_list : Parameter_list
{
    int                         size                        () const                                { return SIZE; }
    const jvalue*               jvalue_array                () const                                { return _jvalues; }

    jvalue                     _jvalues[SIZE];
};


template<>
struct raw_parameter_list<0> : Parameter_list
{
    int size() const { return 0; }
    const jvalue* jvalue_array() const { return NULL; }
};

//--------------------------------------------------------------------------Variable_parameter_list

struct Variable_parameter_list : Parameter_list, Non_cloneable
{
    explicit Variable_parameter_list( int size ) :
        _size( size ),
        _java_values( size ),
        _jvalues ( new jvalue[ size ] )
    {}


    ~Variable_parameter_list() {
        delete[] _jvalues;
    }

    int size() const { return _size; }

    template<typename T>
    void set( int index, const T& v ) 
    {
        if( index >= _size )  throw_xc( Z_FUNCTION, index );

        _java_values[index].assign( v );
        _jvalues[index] = _java_values[index].get_jvalue();
    }
    


    //string signature() 
    //{
    //    string result;
    //    for( uint i = 0; i < _java_values.size(); i++ )
    //        result += _java_values[i].javaType().signature();
    //}


    string signature_string() const;
    const jvalue* jvalue_array() const { return _jvalues; }

private:
    int const _size;
    std::vector<Value> _java_values;
    jvalue* const _jvalues;
};

//---------------------------------------------------------------------------------------Parameters

struct Parameters : Variable_parameter_list
{
    Parameters() : Variable_parameter_list( 0 ) {}


    template<typename T0>
    explicit Parameters( const T0& p0 ) : Variable_parameter_list( 1 ) {
        set( 0, p0 );
    }


    template<typename T0,typename T1>
    Parameters( const T0& p0, const T1& p1 ) : Variable_parameter_list( 2 ) {
        set( 0, p0 );
        set( 1, p1 );}


    template<typename T0,typename T1,typename T2>
    Parameters( const T0& p0, const T1& p1, const T2& p2 ) : Variable_parameter_list( 3 ) {
        set( 0, p0 );
        set( 1, p1 );
        set( 2, p2 );
    }
};

//------------------------------------------------------------------------------------------Jobject
// Neue, vereinfachte Version von Jobject

struct Jobject2 : Object
{
    virtual                    ~Jobject2                    ()                                      {}

                                operator jobject            () const                                { return _jobject; }
    bool                        operator !                  () const                                { return get_jobject() == NULL; }

    jobject                     get_jobject                 () const                                { return _jobject; }
    jobject                     take                        ()                                      { jobject result = _jobject;  _jobject = NULL;  return result; }
    void                        steal_local_ref             ( jobject jo );                         // lo wird danach ungltig!
    virtual void                assign                      ( jobject )                             = 0;    
    void                        construct                   ( const Signature&, const Parameter_list& );
    void                        construct                   ();

    Value                       value_call                  ( const string& method, const Signature&, const Parameter_list& = Parameter_list::empty ) const;


  protected:
                                Jobject2                    ()                                      : _jobject(NULL) {}

  protected:
    jobject                    _jobject;
};

//----------------------------------------------------------------------------------Global_jobject2
// Neue, vereinfachte Version von Global_jobject

struct Global_jobject2 : Jobject2
{
                                Global_jobject2             ()                                      {}
                                Global_jobject2             ( jobject jo )                          { assign( jo ); }
                                Global_jobject2             ( const Global_jobject2& o )            { assign( o.get_jobject() ); }
                               ~Global_jobject2             ()                                      { assign( NULL ); }

    Global_jobject2&            operator =                  ( jobject jo )                          { assign( jo );  return *this; }

    void                        assign                      ( jobject jo );
};

//-------------------------------------------------------------------------------------------------

struct Class_based : Global_jobject2
{
                                Class_based                 ( Class* );
                                Class_based                 ( jobject, Class* );
                                Class_based                 ( const string& class_name );
                                Class_based                 ( const char* class_name ); 

private:
    void                        allocate                    ();

protected:
    ptr<Class>                 _class;
};

//-------------------------------------------------------------------------------------------------

template<class CLASS>
struct class_based : Class_based
{
    class_based( CLASS* clas ) : Class_based( clas ) {}
    class_based( jobject jo, CLASS* clas ) : Class_based( jo, clas ) {}
    class_based( const string& class_name ) : Class_based( Z_NEW( CLASS( class_name ) ) ) {}
    class_based( const char*   class_name ) : Class_based( Z_NEW( CLASS( class_name ) ) ) {}

    CLASS*                      clas                        () const                                { return (CLASS*)+_class; }
};

//------------------------------------------------------------------------------------Class_factory

struct Class_factory
{
                                Class_factory               ( const string& class_name )            : _class_name(class_name) {}

    virtual ptr<Class>          new_instance                () const                                = 0;
    const string&               class_name                  () const                                { return _class_name; }
    Class*                      clas                        () const;

private:
    string const               _class_name;
};

//----------------------------------------------------------------------------------class_factory<>

template<class CLASS>
struct class_factory : Class_factory
{
                                class_factory               ( const string& class_name )            : Class_factory(class_name) {}

    ptr<Class>                  new_instance                () const                                { return +Z_NEW( CLASS( class_name() ) ); }
    CLASS*                      clas                        () const                                { return (CLASS*)Class_factory::clas(); }

private:
    const string               _class_name;
};

//------------------------------------------------------------------------------class_factory_based

template<class CLASS>
struct class_factory_based : class_based<CLASS>
{
                                class_factory_based         ()                                      : class_based<CLASS>( CLASS::class_factory.clas() ) {}
};

//-----------------------------------------------------------------------------------Class_register

struct Class_register
{
    struct Entry {
        const Class_factory*   _class_factory;              // Fr Pointer-Vergleich
        ptr<Class>             _class;
    };


    Class*                      get_or_register_class       ( const Class_factory* );

    template<class CLASS>
    CLASS*                      get_or_register_class       ( const class_factory<CLASS>* f )       { return (CLASS*)get_or_register_class( (Class_factory*)f ); }

  private:
    Class*                      get                         ( const Class_factory* );
    Class*                      register_new_class          ( const Class_factory* );

    Mutex                      _mutex;
    stdext::hash_map<string,Entry> _map;
};

//-------------------------------------------------------------------------------------------------

template<class CLASS>
struct Array : Global_jobject2
{
    //TODO Noch zu implementieren
};

//-------------------------------------------------------------------------------------------------

} //namespace javabridge
} //namespace zschimmer


#endif
