/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: olechar.cxx 4160 2006-03-05 19:13:06Z jz $

#include "system.h"
#include "olechar.h"
#include <stdio.h>   //fprintf
#include <assert.h>


#ifndef Z_HPUX
#   include <wctype.h>
#endif

//-----------------------------------------------------------------------------------------towlower
#ifdef Z_HPUX

int towlower( int c )
{
    return (uint)c < 256? tolower( c ) : c;
}

#endif
//------------------------------------------------------------------------------------------wmemchr

const OLECHAR* wmemchr( const OLECHAR* s, OLECHAR c, int len )
{
    if( s == NULL )
    {
        if( len <= 0 )  return NULL;                    // wmemchr(NULL,c,-1) tolerieren wir.
        fprintf( stderr, "wmemchr(NULL,%d)!\n", len );  // Fhrt zum Absturz
    }

    const OLECHAR* p = s;
    while( (unsigned int)len > 0  &&  *p != c )  p++, len--;        // (uint), damit len=-1 als len=MAX_UINT gedeutet wird.
    return p; 
}

//-------------------------------------------------------------------------------------------wcschr

const OLECHAR* wcschr( const OLECHAR* s, OLECHAR c )
{
    if( !s )  return NULL;

    const OLECHAR* p = s;

    while( *p )
    {
        if( *p == c )  return p;
        p++;
    }

    return NULL;
}

//-------------------------------------------------------------------------------------------wcslen

int wcslen( const OLECHAR* s )
{
    const OLECHAR* p = s;
    if( p )  while( *p )  p++;
    return p - s;
}

//-------------------------------------------------------------------------------------------wcscmp

int wcscmp( const OLECHAR* a, const OLECHAR* b )
{
    if( a == NULL )  return b == NULL  ||  b[0] == '\0'? 0 : -1;
    if( b == NULL )  return +1;

    while( *a   &&  *a == *b )  a++, b++;

    return *a < *b? -1 :
           *a > *b? +1 
                  : 0;

/*
    while( *a | *b )
    {
        if( *a < *b )  return -1;
        if( *a > *b )  return +1;
        a++, b++;
    }

    return 0;
*/
}

//----------------------------------------------------------------------------------------wcscasecmp

int wcscasecmp( const OLECHAR* a, const OLECHAR* b )
{ 
    if( a == NULL )  return b == NULL  ||  b[0] == '\0'? 0 : -1;
    if( b == NULL )  return +1;

    while( *a   &&  towlower(*a) == towlower(*b) )  a++, b++;

    OLECHAR aa = towlower(*a);
    OLECHAR bb = towlower(*b);

    return aa < bb? -1 :
           aa > bb? +1 
                  : 0;
}

//-------------------------------------------------------------------------------------------wcscmp
#ifdef __GNUC__   //  sizeof(OLECHAR) != sizeof(wchar_t)

int wcscmp( const OLECHAR* a, const wchar_t* b )
{
    if( a == NULL )  return b == NULL  ||  b[0] == '\0'? 0 : -1;
    if( b == NULL )  return +1;

    while( *a   &&  *a == *b )  a++, b++;

    return *a < *b? -1 :
           *a > *b? +1 
                  : 0;
}

#endif
//------------------------------------------------------------------------------------------wcsncmp

int wcsncmp( const OLECHAR* a, const OLECHAR* b, int length )
{
    assert( length >= 0 );

    if( length == 0 )  return 0;
    if( a == NULL   )  return b == NULL  ||  b[0] == '\0'? 0 : -1;
    if( b == NULL   )  return +1;

    while( length--  &&  *a   &&  *a == *b )  a++, b++;

    return length == 0? 0  :
           *a < *b    ? -1 :
           *a > *b    ? +1 
                      : 0;
}

//------------------------------------------------------------------------------------------wcsncmp
#ifdef __GNUC__   //  sizeof(OLECHAR) != sizeof(wchar_t)

int wcsncmp( const OLECHAR* a, const wchar_t* b, int length )
{
    assert( length >= 0 );

    if( length == 0 )  return 0;
    if( a == NULL   )  return b == NULL  ||  b[0] == '\0'? 0 : -1;
    if( b == NULL   )  return +1;

    while( length--  &&  *a   &&  *a == *b )  a++, b++;

    return length == 0? 0  :
           *a < *b    ? -1 :
           *a > *b    ? +1 
                      : 0;
}

#endif
//-------------------------------------------------------------------------------------------------

