/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: string_list.cxx 5419 2007-12-06 14:15:42Z jz $

#include "zschimmer.h"
#include "string_list.h"

using namespace std;
using namespace zschimmer::io;


namespace zschimmer {

//------------------------------------------------------------------------------String_list::length

size_t String_list::length() const
{
    size_t result = 0;
    Z_FOR_EACH_CONST( list<string>, _list, it )  result += it->length();
    return result;
}

//---------------------------------------------------------------------------String_list::to_string

string String_list::to_string() const
{
    string result;

    result.reserve( length() );
    Z_FOR_EACH_CONST( list<string>, _list, it )  result += *it;

    return result;
}

//------------------------------------------------------------------String_list::next_char_sequence

Char_sequence String_list::next_char_sequence() const
{
    if( _list.empty() )
    {
        return Char_sequence( NULL, 0 );
    }
    else
    {
        const string* s = &*_list.begin();
        return Char_sequence( s->data() + _read_offset, s->length() - _read_offset );
    }
}

//------------------------------------------------------------------String_list::next_byte_sequence

Byte_sequence String_list::next_byte_sequence() const
{
    if( _list.empty() )
    {
        return Byte_sequence( NULL, 0 );
    }
    else
    {
        const string* s = &*_list.begin();
        return Byte_sequence( (const Byte*)s->data() + _read_offset, s->length() - _read_offset );
    }
}

//---------------------------------------------------------------------------------String_list::eat

void String_list::eat( size_t length )
{
    _read_offset += length;
    assert( _read_offset <= _list.begin()->length() );
    
    if( _read_offset == _list.begin()->length() ) 
    {
        _list.erase( _list.begin() );
        _read_offset = 0;
    }
}

//--------------------------------------------------------------------------String_list::string_eat

string String_list::string_eat( size_t len )
{
    string result;
    result.reserve( min( len, length() ) );

    while( length() )
    {
        Char_sequence seq = next_char_sequence();
        result.append( seq.ptr(), seq.length() );
        eat( seq.length() );
    }

    return result;
}

//--------------------------------------------------------------------String_list::immutable_string

//Immutable_string String_list::to_immutable_string() const
//{
//    Immutable_string_builder result ( length() );
//
//    Z_FOR_EACH_CONST( list<string>, _list, it )  result += *it;
//
//    return result.to_immutable_string();
//}

//-------------------------------------------------------------------------------------------------

} //namespace zschimmer

