/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: system_command.cxx 5384 2007-11-14 18:00:23Z jz $
//  2004 Joacim Zschimmer

#include "zschimmer.h"
#include "log.h"
#include "system_command.h"
#include "file.h"

#include <stdio.h>
#include <sys/types.h>
#include <sys/timeb.h>

#ifdef Z_WINDOWS
#   include <windows.h>
#   include <io.h>
#else
#   include <unistd.h>
#   include <locale.h>
#   include <sys/time.h>
#   include <signal.h>
#endif

using namespace std;
using namespace zschimmer::file;


namespace zschimmer {

//--------------------------------------------------------------------------System_command::execute

/* Neuer Code fr Windows, der das Kommando direkt (nicht ber cmd.exe) ruft und deshalb die Anfhrungszeichen korrekt bergibt: 
   2007-11-14

void System_command::execute( const string& cmd )
{
    File   stderr_file;
    File   stdout_file;
    string my_cmd = cmd;

    _stderr_text = "";

    //if( cmd.find( "2>" ) == string::npos )
    //{
          stderr_file.create_temporary( File::open_unlink_later );
    //    my_cmd.append( " 2>" + stderr_file.path() );
    //}

    //
    //if( !has_regex( cmd, "[^2]>" ) )
    //{
          stdout_file.create_temporary( File::open_unlink_later );
    //    my_cmd.append( " >" + stdout_file.path() );
    //}



#   ifdef Z_WINDOWS

        // Wir nehmen nicht system(), weil das in Anfhrungszeichen gesetzte Parameter nicht korrekt bergibt.

        Process process;
        process.set_stdout_path( stdout_file.path() );
        process.set_stderr_path( stderr_file.path() );

        process.start( my_cmd );
        process.wait();
        _exit_code = process.exit_code();

#    else
        my_cmd.append( " 2>" + stderr_file.path() );
        my_cmd.append( " >"  + stdout_file.path() );

        Z_LOG( "signal(SIGCHLD,SIG_DFL)\n" );
        ::signal( SIGCHLD, SIG_DFL );                 // 2006-03-04 Fr Suse 9.2  (Java verndert das Signal-Verhalten, so dass waitpid() ohne diesen Aufruf versagte.)

        Z_LOG( "system " << my_cmd << "\n" );

        errno = 0;
        int ret = system( my_cmd.c_str() );
        int system_errno = errno;

        _exit_code = ret == -1? -1 : WEXITSTATUS(ret);

#   endif


    if( stderr_file.path() != "" )
    {
        try {
            _stderr_text = string_from_file( stderr_file.path() );
        }
        catch( const exception& ) {}
    }

    if( stdout_file.path() != "" )
    {
        try {
            _stdout_text = string_from_file( stdout_file.path() );
        }
        catch( const exception& ) {}
    }

    try
    {
#       ifdef Z_WINDOWS
            if( _exit_code != 0 )  throw_xc( "Z-4005", as_string(_exit_code), cmd );
#        else
            if( ret == -1 )  throw_errno( system_errno, "system" );
            if( WIFSIGNALED(ret) )                      throw_xc( "Z-4006", as_string(WTERMSIG(ret)), cmd );
            if( WIFEXITED(ret)  &&  WEXITSTATUS(ret) )  throw_xc( "Z-4005", as_string(WEXITSTATUS(ret)), cmd );
#       endif
    }
    catch( const Xc& x )
    {
        _error = true;
        if( _throw_xc )  throw;
        _xc = x;
    }
}
*/

void System_command::execute( const string& cmd )
{
    File stderr_file;
    File stdout_file;
    string my_cmd = cmd;

    _stderr_text = "";

    if( cmd.find( "2>" ) == string::npos )
    {
        stderr_file.create_temporary( File::open_unlink_later );
        my_cmd.append( " 2>" + stderr_file.path() );
    }

    
    if( !has_regex( cmd, "[^2]>" ) )
    {
        stdout_file.create_temporary( File::open_unlink_later );
        my_cmd.append( " >" + stdout_file.path() );
    }


#   ifdef Z_UNIX
        Z_LOG( "signal(SIGCHLD,SIG_DFL)\n" );
        ::signal( SIGCHLD, SIG_DFL );                 // 2006-03-04 Fr Suse 9.2  (Java verndert das Signal-Verhalten, so dass waitpid() ohne diesen Aufruf versagte.)
#   endif


    Z_LOG( "system " << my_cmd << "\n" );
    errno = 0;
    int ret = system( my_cmd.c_str() );
    int system_errno = errno;

#   ifdef Z_WINDOWS
        _exit_code = ret;
#    else
        _exit_code = ret == -1? -1 : WEXITSTATUS(ret);
#   endif


    if( stderr_file.path() != "" )
    {
        try {
            _stderr_text = string_from_file( stderr_file.path() );
        }
        catch( const exception& ) {}

        //unlink( stderr_file.filename().c_str() );
    }

    if( stdout_file.path() != "" )
    {
        try {
            _stdout_text = string_from_file( stdout_file.path() );
        }
        catch( const exception& ) {}

        //unlink( stdout_file.filename().c_str() );
    }

    try
    {
        if( ret == -1 )  throw_errno( system_errno, "system" );

#       ifdef Z_WINDOWS
            if( ret != 0 )  throw_xc( "Z-4005", as_string(ret), cmd );
#        else
            if( WIFSIGNALED(ret) )                      throw_xc( "Z-4006", as_string(WTERMSIG(ret)), cmd );
            if( WIFEXITED(ret)  &&  WEXITSTATUS(ret) )  throw_xc( "Z-4005", as_string(WEXITSTATUS(ret)), cmd );
#       endif
    }
    catch( const Xc& x )
    {
        _error = true;
        if( _throw_xc )  throw;
        _xc = x;
    }
}

//-------------------------------------------------------------------------------------------------

} //namespace zschimmer
