/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: z_posix.cxx 5477 2008-01-04 14:47:23Z jz $

#include "zschimmer.h"
#include "z_posix.h"
#include "mutex.h"


#ifdef Z_UNIX

#include <unistd.h>
#include <signal.h>
#include <math.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <sys/socket.h>

#ifdef Z_SOLARIS
#   include <stropts.h>
#   include <sys/filio.h>       // ioctl( , FIONBIO )
#endif

using namespace std;
using namespace zschimmer;

//-------------------------------------------------------------------------------------------static

static posix::Undestroyable_mutex interlocked_increment_mutex ( "InterlockedIncrement", Mutex::kind_nonrecursive );    // Fr InterlockedIncrement() und InterlockedDecrement()

//-----------------------------------------------------------------------------InterlockedIncrement

long32 InterlockedIncrement( long32* i )  
{ 
    long32 result = 0;

    //Z_MUTEX( interlocked_increment_mutex )
    {
        interlocked_increment_mutex.enter();
        (*i)++; 
        result = *i;
        interlocked_increment_mutex.leave();
    }

    return result;
}

//-----------------------------------------------------------------------------InterlockedDecrement

long32 InterlockedDecrement( long32* i )  
{ 
    long result = 0;

    //Z_MUTEX( interlocked_increment_mutex )
    {
        interlocked_increment_mutex.enter();
        (*i)--; 
        result = *i; 
        interlocked_increment_mutex.leave();
    }

    return result;
}

//---------------------------------------------------------------------------------------strerror_s

errno_t strerror_s( char* buffer, size_t buffer_size, errno_t errn )
{
    errno_t result = 0;

    if( buffer_size > 0 )  buffer[0] = '\0';

#   if defined Z_AIX || defined Z_SOLARIS || defined Z_HPUX
        strncpy( buffer, strerror( errn ), buffer_size );
#   elif 0  // Warum liefert strerror_r() nicht int?
        int ret = strerror_r( errn, buffer, buffer_size );
        result = ret == 0? 0 : errno;
#   else
        const char* p = strerror_r( errn, buffer, buffer_size );
        if( p && p != buffer )  strncpy( buffer, p, buffer_size );
        result = p == NULL? 0 : errno;
#   endif

    if( buffer_size > 0 )  buffer[ buffer_size - 1 ] = '\0';
    
    return result;
}

//-------------------------------------------------------------------------------------------------

namespace zschimmer {
namespace posix {

//-------------------------------------------------------------------------------------------------

void timespec_add( timespec* ts, double seconds )
{
    ts->tv_sec = (time_t)floor( seconds );
    ts->tv_nsec = (time_t)max( 0.0, ( seconds - floor( seconds ) ) * 1e9 );
}

//-------------------------------------------------------------------------------------------------

void timespec_add( timespec* ts, const timeval& tv )
{
    int u = ts->tv_nsec / 1000 + tv.tv_usec;
    ts->tv_nsec = u % 1000000 * 1000  + ts->tv_nsec % 1000;
    ts->tv_sec += tv.tv_sec + u / 1000000;
}

//-------------------------------------------------------------------------------------------------

} //namespace posix
} //namespace zschimmer

#endif
